
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "application.h"
#include <QtDebug>
#include <QFile>

Application::Application(const QString & responseString)
        : QObject(),
        p_largeIcon(0),
        p_smallIcon(0),
        p_cache(responseString),
	p_enabled(true)
{
    // Getting params list from the responseString
    QStringList appParams = responseString.split(",");
    //qDebug() << responseString;

    // Parsing parameters
    foreach (QString param, appParams)
    {
        // Name of application
        if (param.startsWith("Name=", Qt::CaseInsensitive))
        {
            p_name = param;
            p_name.remove("Name=");
            continue;
        }

        // Folder of application
        if (param.startsWith("FolderName=", Qt::CaseInsensitive))
        {
            p_folder = param;
            p_folder.remove("FolderName=");
            continue;
        }

        // Small icon for application
        //
        // TODO: Need to put in a means to use a default icon
        //       in place of an empty one.
        //
        if (param.startsWith("SmallIcon=", Qt::CaseInsensitive))
        {
            QString appSmallIcon = param;
            appSmallIcon.remove("SmallIcon=");
            // Loading small icon from the temp file
            p_smallIcon = fileToPixmap(appSmallIcon);
            // Icon was load successfully
            if (p_smallIcon != 0)
                p_smallIconName = appSmallIcon;
            continue;
        }

        // Large icon for application
        if (param.startsWith("LargeIcon=", Qt::CaseInsensitive))
        {
            QString appLargeIcon = param;
            appLargeIcon.remove("LargeIcon=");
            // Loading large icon from the temp file
            p_largeIcon = fileToPixmap(appLargeIcon);
            // Icon was load successfully
            if (p_largeIcon != 0)
                p_largeIconName = appLargeIcon;
            continue;
        }

        //Enable/disable item
        if (param.startsWith("Enabled=", Qt::CaseInsensitive))
        {
            QString enabled = param;
            enabled.remove("Enabled=");
            p_enabled = enabled.startsWith("true", Qt::CaseInsensitive);
            continue;
        }
    }
}

QString Application::name() const
{
    return p_name;
}

QString Application::folder() const
{
    return p_folder;
}

QString Application::cache() const
{
    return p_cache;
}

QPixmap * Application::smallIcon() const
{
    return p_smallIcon;
}

QPixmap * Application::largeIcon() const
{
    return p_largeIcon;
}

void Application::clearCache()
{
    // Clearing temporary application files
    //
    // DCT: Adding check to make sure that the files are valid to begin with
    //
    if ( !p_smallIconName.isNull() && !p_smallIconName.isEmpty() )
    {
        QFile smallIconFile(p_smallIconName);
        smallIconFile.remove();
    }

    if ( !p_largeIconName.isNull()  && !p_smallIconName.isEmpty())
    {
        QFile largeIconFile(p_largeIconName);
        largeIconFile.remove();
    }
}

bool Application::isEnabled()
{
    return p_enabled;
}
