
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "autoLaunchPage.h"

#include <QListWidgetItem>

AutoLaunchPage::AutoLaunchPage(Farm *farm, ManageConnectionDialog *parent)
    : ManageConnectionPage(farm, parent)
{
        setupUi(this);

        applicationsListWidget->setVisible(false);
        addButton->setVisible(false);

        connect(addButton, SIGNAL(clicked()), SLOT(addApplication()));
	connect(browseButton, SIGNAL(clicked()), SLOT(browseApplication()));
	connect(deleteButton, SIGNAL(clicked()), SLOT(removeApplication()));
	connect(upButton, SIGNAL(clicked()), SLOT(upApplication()));
	connect(downButton, SIGNAL(clicked()), SLOT(downApplication()));
}

AutoLaunchPage::~AutoLaunchPage()
{

}

void AutoLaunchPage::addApplication()
{
    QListWidgetItem * addItem = applicationsListWidget->currentItem();
    if (addItem == 0)
        return;

    QString addName = addItem->text();
    if (autolaunchListWidget->findItems(addName, Qt::MatchFixedString).isEmpty())
    {
        QListWidgetItem * newItem = addItem->clone();
        autolaunchListWidget->addItem(newItem);
    }
}

void AutoLaunchPage::removeApplication()
{
    if (autolaunchListWidget->currentItem() == 0)
        return;
    autolaunchListWidget->takeItem(autolaunchListWidget->currentRow());
}

void AutoLaunchPage::upApplication()
{
    int currentIndex = autolaunchListWidget->currentRow();
    if (currentIndex > 0)
    {
        QListWidgetItem *tempItem = autolaunchListWidget->takeItem(currentIndex);
        autolaunchListWidget->insertItem(currentIndex - 1, tempItem);
        autolaunchListWidget->setCurrentItem(tempItem);
    }
    autolaunchListWidget->setFocus();
}

void AutoLaunchPage::downApplication()
{
    int currentIndex = autolaunchListWidget->currentRow();
    if (currentIndex < autolaunchListWidget->count())
    {
        QListWidgetItem *tempItem = autolaunchListWidget->takeItem(currentIndex);
        autolaunchListWidget->insertItem(currentIndex + 1, tempItem);
        autolaunchListWidget->setCurrentItem(tempItem);
    }
    autolaunchListWidget->setFocus();
}

void AutoLaunchPage::browseApplication()
{
    applicationsListWidget->setVisible(!applicationsListWidget->isVisible());
    addButton->setVisible(!addButton->isVisible());
}

void AutoLaunchPage::saveFarmSettings()
{
    // Farm doesn`t existing
    if (currentFarm == 0)
        return;

    // Saving farm settings
    ApplicationList apps = currentFarm->applications();
    ApplicationList autolaunch;
    for (int i = 0; i < autolaunchListWidget->count(); i++)
    {
        QListWidgetItem * item = autolaunchListWidget->item(i);
        QString name = item->text();

        Application * app = apps.application(name);
        if (app != 0)
        {
            autolaunch << app;
//            qDebug() << "Next name." << app->name();
        }
    }

    currentFarm->setAutolaunch(autolaunch);
}

void AutoLaunchPage::loadFarmSettings()
{
    // Farm doesn`t existing
    if (currentFarm == 0)
        return;

    applicationsListWidget->clear();
    ApplicationList apps = currentFarm->applications();
    foreach (Application * app, apps)
    {
        QListWidgetItem * item = new QListWidgetItem;
        // Only do this if the icon is not empty
        if ( app->smallIcon() != NULL )
        {
            // fprintf( stderr, "WARNING: Empty small icon object for app: %s.\n", app->name().toAscii().data());
            item->setIcon(*(app->smallIcon()));
            item->setText(app->name());
            applicationsListWidget->addItem(item);
        }
    }

    if (apps.isEmpty())
        browseButton->setEnabled(false);

    autolaunchListWidget->clear();
    ApplicationList autolaunch = currentFarm->autolaunch();
    QStringList autolaunchStringList = autolaunch.names();
    foreach (QString appName, autolaunchStringList)
    {
        Application * app = apps.application(appName);
        if (app != 0)
        {
            QListWidgetItem * item = new QListWidgetItem;
            item->setIcon(*(app->smallIcon()));
            item->setText(app->name());
            autolaunchListWidget->addItem(item);
        }
    }
}

//void AutoLaunchPage::updateApplicationList(QStringList autoLaunchList)
//{
//    // Searching for new items
//    foreach (QString name, autoLaunchList)
//    {
//        // Find items with this name
//        QList<QListWidgetItem *> items = applicationsListWidget->findItems(name,
//                                                                           Qt::MatchCaseSensitive);
//
//        // Add if unique name
//        if (items.isEmpty())
//            applicationsListWidget->addItem(name);
//    }
//
//    // Searching for items needs to be removed
//    for (int i = 0; i < applicationsListWidget->count(); i++)
//    {
//        // Getting item
//        QListWidgetItem *item = applicationsListWidget->item(i);
//
//        // Getting name of the item
//        QString name = item->text();
//
//        // Searching for name
//        if (autoLaunchList.indexOf(name) < 0)
//            applicationsListWidget->takeItem(i);
//    }
//}
