
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "firewallProxyPage.h"
#include <QtDebug>

FirewallProxyPage::FirewallProxyPage(Farm *farm, ManageConnectionDialog *parent)
    : ManageConnectionPage(farm, parent)
{
	setupUi(this);
	proxyButtonGroup = new QButtonGroup;
	proxyButtonGroup->addButton(defualtProxyRadioButton);
	proxyButtonGroup->addButton(dontUseProxyRadioButton);
	proxyButtonGroup->addButton(manuallyProxyRadioButton);
	proxyButtonGroup->setExclusive(true);

	connect(locationComboBox, SIGNAL(currentIndexChanged(int)),
			SLOT(currentLocationChanged(int)));
	connect(sslEnabledCheckBox, SIGNAL(toggled(bool)), SLOT(enableSslGateway(bool)));
	connect(proxyButtonGroup, SIGNAL(buttonClicked(QAbstractButton *)),
			SLOT(currentProxyOptionChanged(QAbstractButton *)));
}

FirewallProxyPage::~FirewallProxyPage()
{

}

void FirewallProxyPage::updateLocationName(const QString & text)
{
	connectionOptionsGroupBox->setTitle(tr("Connection Options for Location [") + text + "]");
	proxyServerGroupBox->setTitle(tr("Proxy Server for Location [") + text + "]");
}

void FirewallProxyPage::enableSslGateway(bool enableSslGateway)
{
	sslGatewayLabel->setEnabled(enableSslGateway);
        sslGatewayLineEdit->setEnabled(enableSslGateway);
}

void FirewallProxyPage::currentProxyOptionChanged(QAbstractButton *button)
{
    Location *location = currentFarm->getCurrentLocation();
    // No current location
    if (location == 0)
            return;

	if (button == defualtProxyRadioButton)
	{
        proxyLabel->setEnabled(false);
        proxyLineEdit->setEnabled(false);
		dontUseProxyLabel->setEnabled(false);
		descProxyLabel->setEnabled(false);
		dontUseProxyLineEdit->setEnabled(false);
		proxyLineEdit->setText("");
        location->setProxyServer(proxyLineEdit->text());
        location->setProxyBypassList(dontUseProxyLineEdit->text());
	}
	if (button == dontUseProxyRadioButton)
	{
		proxyLabel->setEnabled(false);
		proxyLineEdit->setEnabled(false);
		dontUseProxyLabel->setEnabled(false);
		descProxyLabel->setEnabled(false);
		dontUseProxyLineEdit->setEnabled(false);
		proxyLineEdit->setText("");
        location->setProxyServer(proxyLineEdit->text());
        location->setProxyBypassList(dontUseProxyLineEdit->text());
	}
	if (button == manuallyProxyRadioButton)
	{
		proxyLabel->setEnabled(true);
		proxyLineEdit->setEnabled(true);
		dontUseProxyLabel->setEnabled(true);
		descProxyLabel->setEnabled(true);
		dontUseProxyLineEdit->setEnabled(true);
	}
}

void FirewallProxyPage::saveLocationSettings()
{
    // No current farm
    if (currentFarm == 0)
        return;

        Location *location = currentFarm->getCurrentLocation();
        // No current location
        if (location == 0)
                return;

        location->setNat(natEnabledCheckBox->isChecked());
        bool isSsl = sslEnabledCheckBox->isChecked();
        location->setSsl(isSsl);
        if (isSsl)
            location->setSslGateway(sslGatewayLineEdit->text());
        else
            location->setSslGateway("");

    if (proxyLineEdit->isEnabled())
    {
        location->setProxyServer(proxyLineEdit->text());
        location->setProxyBypassList(dontUseProxyLineEdit->text());
    }
}

void FirewallProxyPage::loadLocationSettings()
{
        // No current farm
        if (currentFarm == 0)
                return;

        int currentLocationIndex = locationComboBox->currentIndex();
        Location *location = currentFarm->getLocation(currentLocationIndex);
        // No current location
        if (location == 0)
                return;

        natEnabledCheckBox->setChecked(location->isNat());

	sslGatewayLineEdit->setText(location->getSslGateway());
	sslEnabledCheckBox->setChecked(location->isSsl());

        QString proxyServer = location->getProxyServer();
        QString proxyBypassList = location->getProxyBypassList();
        proxyLineEdit->setText(proxyServer);
        dontUseProxyLineEdit->setText(proxyBypassList);
        if (proxyServer.isEmpty() && proxyBypassList.isEmpty())
        {
            dontUseProxyRadioButton->setChecked(true);
            this->currentProxyOptionChanged(dontUseProxyRadioButton);
        }
        if (proxyServer.isEmpty() && !proxyBypassList.isEmpty())
        {
            defualtProxyRadioButton->setChecked(true);
            this->currentProxyOptionChanged(defualtProxyRadioButton);
        }
        if (!proxyServer.isEmpty() && proxyBypassList.isEmpty())
        {
            manuallyProxyRadioButton->setChecked(true);
            this->currentProxyOptionChanged(manuallyProxyRadioButton);
        }
}

void FirewallProxyPage::updateExistingLocationsList()
{
    // No existing farm
    if (currentFarm == 0)
            return;

    disconnect(locationComboBox, SIGNAL(currentIndexChanged(int)),
            this, SLOT(currentLocationChanged(int)));
    // Creating locations list
    QStringList locationsList = currentFarm->getLocationNamesList();
    locationComboBox->clear();
    locationComboBox->addItems(locationsList);
    connect(locationComboBox, SIGNAL(currentIndexChanged(int)),
            SLOT(currentLocationChanged(int)));

    // Loading current location
    int currentIndex = currentFarm->getLastLocation();
    locationComboBox->setCurrentIndex(currentIndex);
    currentFarm->setCurrentLocation(currentIndex);
    this->loadLocationSettings();
    this->updateLocationName(locationComboBox->currentText());
}

void FirewallProxyPage::currentLocationChanged(int index)
{
   if (currentFarm == 0)
        return;

    // Break if index is incorrect
    if (index < 0 )
    {
        currentFarm->setCurrentLocation(0);
        return;
    }

    // Save current location settings
    Location *currentLocation = currentFarm->getCurrentLocation();
    if (currentLocation != 0 &&
        currentLocation->getName() != locationComboBox->currentText())
        this->saveLocationSettings();

    // Change current location
    currentFarm->setCurrentLocation(index);

    // Load current location settings
    this->loadLocationSettings();

    // Update current location name on form
    this->updateLocationName(locationComboBox->currentText());
}

void FirewallProxyPage::loadFarmSettings()
{
    // Farm doesn`t existing
    if (currentFarm == 0)
        return;

    bool settingsHided = currentFarm->isHideSettings();
    if (settingsHided)
        firewallWidget->setCurrentIndex(0);
    else
        firewallWidget->setCurrentIndex(1);

    // Update existing locations list
    this->updateExistingLocationsList();
}

void FirewallProxyPage::saveFarmSettings()
{
    if (currentFarm == 0)
        return;

    if (currentFarm->getCurrentLocation() != 0)
        this->saveLocationSettings();
}

/*virtual*/ void FirewallProxyPage::showEvent(QShowEvent *)
{
    if (currentFarm == 0)
        return;

    // Update existing locations list
    this->updateExistingLocationsList();
}

/*virtual*/ void FirewallProxyPage::hideEvent(QHideEvent *)
{
      if (currentFarm == 0)
        return;

    int currentIndex = locationComboBox->currentIndex();

    // If no index
    if (currentIndex < 0)
        return;

    currentFarm->setLastLocation(currentIndex);

    if (currentFarm->getCurrentLocation() != 0)
        this->saveLocationSettings();
}
