
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "crypto.h"
#include "rc4.h"

static unsigned char scatter256[] = 
{
    0x80, 0x5b, 0xe8, 0x18, 0x6f, 0x64, 0x89, 0x3a, 0x34, 0xce, 0x59, 0xdf, 0x4d, 0xb4, 0x5a, 0x0f,
    0x69, 0x94, 0x58, 0x70, 0x71, 0x4b, 0x17, 0xcf, 0xc3, 0x40, 0xaa, 0xfc, 0xc5, 0xe0, 0x21, 0xdb,
    0x9a, 0x49, 0x68, 0xb8, 0x2f, 0x4a, 0x6c, 0xdc, 0x7a, 0x8b, 0x7f, 0x5c, 0x03, 0x08, 0xfe, 0x39,
    0xa3, 0xc6, 0x31, 0xa6, 0x8c, 0xbd, 0x72, 0xa4, 0x8a, 0x1b, 0x92, 0xd5, 0x87, 0xad, 0x78, 0x8f,
    0x55, 0x96, 0x0b, 0x30, 0xa8, 0x43, 0x53, 0xb0, 0x62, 0xa0, 0xda, 0x7c, 0x13, 0x8d, 0x5d, 0x81,
    0xc0, 0x8e, 0x90, 0x88, 0xe4, 0xb7, 0x76, 0xc2, 0xb5, 0x04, 0x93, 0xa5, 0xa9, 0x9e, 0xab, 0xf5,
    0x37, 0xac, 0x99, 0x26, 0xe2, 0x38, 0x85, 0xe1, 0x74, 0x77, 0x32, 0xe5, 0x91, 0x23, 0xb1, 0x10,
    0x4c, 0x47, 0x3f, 0xbe, 0x82, 0x22, 0x6a, 0x51, 0xd0, 0x63, 0x75, 0x11, 0x33, 0x9b, 0xfb, 0x3b,
    0xca, 0xed, 0xdd, 0x44, 0xe6, 0x12, 0x4e, 0x97, 0x3c, 0x79, 0x4f, 0x41, 0x66, 0xba, 0x50, 0x0e,
    0xc9, 0x6b, 0x05, 0xee, 0x6e, 0xe7, 0x95, 0x7b, 0x60, 0x9d, 0xff, 0xc4, 0x29, 0x86, 0xb9, 0x7d,
    0x98, 0xc8, 0x9c, 0x35, 0xbb, 0xbc, 0xef, 0xfa, 0x3d, 0x06, 0xf9, 0x36, 0xbf, 0x3e, 0x7e, 0xa2,
    0xc7, 0x56, 0xae, 0xcb, 0xaf, 0xe9, 0x42, 0x61, 0xf0, 0x1d, 0xfd, 0x65, 0x9f, 0x52, 0x27, 0xea,
    0x24, 0xa1, 0xa7, 0xb2, 0x6d, 0x14, 0xb3, 0x45, 0xf8, 0xb6, 0xf7, 0x73, 0xc1, 0x83, 0x84, 0xf4,
    0xcc, 0xcd, 0xf3, 0xe3, 0x54, 0x15, 0xd1, 0x46, 0x07, 0x57, 0x2c, 0xd2, 0xd3, 0xd6, 0xd4, 0xd7,
    0xf6, 0xeb, 0xd8, 0x1c, 0x00, 0x09, 0xec, 0x67, 0x0a, 0xd9, 0x16, 0xde, 0xf1, 0xf2, 0x01, 0x2d,
    0x5e, 0x48, 0x02, 0x0c, 0x5f, 0x0d, 0x19, 0x1a, 0x28, 0x1e, 0x1f, 0x20, 0x25, 0x2a, 0x2b, 0x2e
};

static unsigned short scatter1024[] = 
{
    0x03c8, 0x0260, 0x023d, 0x03e7, 0x0299, 0x0124, 0x008f, 0x0057, 0x0154, 0x020e, 0x02bf, 0x023b, 0x008d, 0x02f1, 0x00a6, 0x0215,
    0x028e, 0x0185, 0x036d, 0x0180, 0x0354, 0x035c, 0x01aa, 0x02c3, 0x01db, 0x0217, 0x02cb, 0x02a6, 0x029c, 0x03d9, 0x01b8, 0x0085,
    0x0353, 0x03fc, 0x02b4, 0x00a2, 0x0005, 0x0003, 0x0084, 0x00bf, 0x0142, 0x00d0, 0x00d4, 0x02ed, 0x03a8, 0x0018, 0x03d6, 0x029f,
    0x00a8, 0x0213, 0x032c, 0x0002, 0x008e, 0x0378, 0x0008, 0x0191, 0x01b9, 0x03a9, 0x0092, 0x03e4, 0x0132, 0x0329, 0x008a, 0x03c9,
    0x005e, 0x0257, 0x012e, 0x0295, 0x000b, 0x0056, 0x005f, 0x00bc, 0x00b2, 0x024b, 0x02fe, 0x02a0, 0x03fd, 0x02cc, 0x029d, 0x03a7,
    0x0386, 0x0195, 0x0252, 0x0390, 0x00e2, 0x037c, 0x02ee, 0x0207, 0x02de, 0x02a7, 0x0251, 0x0076, 0x0009, 0x02fc, 0x0216, 0x021f,
    0x0272, 0x02d9, 0x0173, 0x0268, 0x00b3, 0x0006, 0x0263, 0x0175, 0x002d, 0x00e7, 0x0204, 0x02fd, 0x0096, 0x01f6, 0x003f, 0x0355,
    0x00b1, 0x0371, 0x02a8, 0x02d1, 0x005d, 0x0350, 0x02a4, 0x034d, 0x03d1, 0x037a, 0x02d2, 0x0007, 0x0026, 0x0038, 0x03a2, 0x03b0,
    0x0116, 0x02ea, 0x034a, 0x00c2, 0x0004, 0x00f9, 0x00dc, 0x0013, 0x003a, 0x030c, 0x02b0, 0x01ab, 0x007a, 0x007f, 0x000a, 0x03d2,
    0x01b0, 0x0311, 0x01f3, 0x006e, 0x0305, 0x02dc, 0x0372, 0x008c, 0x031a, 0x028b, 0x00da, 0x023e, 0x0292, 0x03c7, 0x0279, 0x00a3,
    0x03d3, 0x03f2, 0x027c, 0x024c, 0x000c, 0x0118, 0x0210, 0x01bd, 0x0162, 0x0322, 0x01c8, 0x0052, 0x02b1, 0x014d, 0x013f, 0x0346,
    0x000d, 0x03db, 0x01fe, 0x030e, 0x00e3, 0x0209, 0x01a0, 0x00ec, 0x0342, 0x0040, 0x0334, 0x02c0, 0x0058, 0x028f, 0x03e2, 0x0121,
    0x022e, 0x0359, 0x02d3, 0x03ac, 0x02c1, 0x024d, 0x0049, 0x039c, 0x002c, 0x0391, 0x0015, 0x009c, 0x0244, 0x034b, 0x032d, 0x03da,
    0x014b, 0x0139, 0x0090, 0x015a, 0x01fd, 0x00bd, 0x0176, 0x0394, 0x02d0, 0x0103, 0x00a7, 0x013a, 0x027a, 0x017a, 0x0125, 0x0152,
    0x03b3, 0x0287, 0x0011, 0x018d, 0x023c, 0x027b, 0x01cd, 0x02d8, 0x0120, 0x02c2, 0x025f, 0x002e, 0x023a, 0x015c, 0x0117, 0x03b1,
    0x02f7, 0x0293, 0x0370, 0x01fa, 0x025d, 0x02e1, 0x033a, 0x03ad, 0x004c, 0x013b, 0x03f9, 0x0150, 0x0205, 0x016f, 0x0389, 0x015d,
    0x03e8, 0x02e5, 0x0206, 0x0095, 0x0082, 0x038e, 0x00e6, 0x0098, 0x00c6, 0x031b, 0x016e, 0x00b4, 0x019b, 0x026d, 0x0247, 0x00fa,
    0x0297, 0x034e, 0x0067, 0x0395, 0x020a, 0x0060, 0x0338, 0x0061, 0x023f, 0x014f, 0x03f5, 0x02ac, 0x03fe, 0x0356, 0x0059, 0x016a,
    0x0255, 0x01d9, 0x0072, 0x016d, 0x0399, 0x036e, 0x03dc, 0x0208, 0x03aa, 0x0306, 0x020b, 0x03d4, 0x036f, 0x0164, 0x020c, 0x01d7,
    0x01b4, 0x02d4, 0x013d, 0x00d3, 0x010d, 0x011a, 0x01b5, 0x00d1, 0x0330, 0x01a9, 0x0362, 0x00f6, 0x0170, 0x0016, 0x00d8, 0x01a3,
    0x0283, 0x00cc, 0x0323, 0x00be, 0x0189, 0x0001, 0x00f1, 0x0046, 0x024a, 0x00e9, 0x00fb, 0x0331, 0x02c4, 0x0129, 0x039a, 0x0073,
    0x01d5, 0x0091, 0x03ba, 0x0062, 0x0373, 0x00ff, 0x02f9, 0x002a, 0x01a7, 0x02b2, 0x0108, 0x03d5, 0x0360, 0x0099, 0x0253, 0x022f,
    0x03fa, 0x0127, 0x01dd, 0x032e, 0x005a, 0x0332, 0x0275, 0x0083, 0x0137, 0x032f, 0x0307, 0x0079, 0x0294, 0x00a4, 0x024e, 0x02f6,
    0x0086, 0x03e3, 0x01a5, 0x00db, 0x0133, 0x03f4, 0x034f, 0x0196, 0x002f, 0x00cd, 0x03af, 0x02f2, 0x02da, 0x01c6, 0x020d, 0x0333,
    0x0240, 0x0050, 0x006b, 0x0163, 0x0218, 0x02e4, 0x02ad, 0x03e6, 0x00f7, 0x003b, 0x00fc, 0x0155, 0x01f1, 0x02bd, 0x0165, 0x0388,
    0x0343, 0x003c, 0x00c8, 0x00f5, 0x0265, 0x01e0, 0x02fa, 0x0039, 0x0148, 0x0065, 0x021e, 0x01f4, 0x00df, 0x0087, 0x0153, 0x00dd,
    0x00de, 0x03b2, 0x0296, 0x000e, 0x00c3, 0x0104, 0x03df, 0x0396, 0x0211, 0x027d, 0x0397, 0x0368, 0x03e5, 0x025c, 0x0028, 0x0051,
    0x03a1, 0x00fe, 0x02e9, 0x0352, 0x030d, 0x02ae, 0x0246, 0x033d, 0x0382, 0x01e3, 0x031e, 0x0088, 0x027e, 0x01be, 0x016b, 0x0351,
    0x035d, 0x00b0, 0x0020, 0x03c6, 0x0267, 0x0379, 0x005b, 0x00b8, 0x010e, 0x0053, 0x03ab, 0x0063, 0x0374, 0x0269, 0x01e4, 0x0377,
    0x0324, 0x0194, 0x01d1, 0x029a, 0x0335, 0x0242, 0x017b, 0x01ca, 0x0166, 0x01af, 0x0177, 0x024f, 0x00bb, 0x0259, 0x039b, 0x02b3,
    0x0144, 0x00b7, 0x00d6, 0x0143, 0x010f, 0x0029, 0x025a, 0x0078, 0x0274, 0x002b, 0x01fb, 0x01de, 0x03a0, 0x01cc, 0x01dc, 0x0325,
    0x0357, 0x0167, 0x0348, 0x0178, 0x02dd, 0x0089, 0x00e0, 0x0308, 0x0172, 0x0030, 0x03f1, 0x02ef, 0x00a0, 0x033e, 0x012f, 0x0336,
    0x021c, 0x012d, 0x0080, 0x0128, 0x02be, 0x00fd, 0x0031, 0x0239, 0x02cd, 0x006c, 0x0358, 0x018e, 0x037d, 0x0375, 0x035b, 0x0181,
    0x01b6, 0x03dd, 0x0119, 0x0290, 0x0214, 0x0168, 0x00b9, 0x021d, 0x0237, 0x01ce, 0x0156, 0x0219, 0x0337, 0x0157, 0x036b, 0x00f4,
    0x016c, 0x037b, 0x02e3, 0x001c, 0x0182, 0x03de, 0x0023, 0x006d, 0x02a1, 0x037e, 0x0074, 0x011b, 0x020f, 0x0234, 0x01a8, 0x00b5,
    0x03d7, 0x005c, 0x0102, 0x008b, 0x00e1, 0x00c1, 0x0158, 0x0233, 0x0241, 0x02f0, 0x0068, 0x036c, 0x0183, 0x0187, 0x039d, 0x0021,
    0x02bc, 0x0105, 0x03ca, 0x02c8, 0x035e, 0x02b5, 0x007b, 0x02f3, 0x0276, 0x03cb, 0x0229, 0x0225, 0x0256, 0x010a, 0x000f, 0x02e6,
    0x0339, 0x0250, 0x00d5, 0x0212, 0x034c, 0x0286, 0x02f8, 0x0273, 0x0010, 0x0000, 0x03f3, 0x0298, 0x0093, 0x00ba, 0x0302, 0x00e4,
    0x0197, 0x033b, 0x02ff, 0x01d3, 0x0258, 0x0361, 0x037f, 0x00b6, 0x02d5, 0x03b7, 0x013e, 0x025b, 0x0363, 0x02d6, 0x01cb, 0x0243,
    0x0169, 0x011c, 0x025e, 0x01cf, 0x015e, 0x029b, 0x01eb, 0x0202, 0x0220, 0x0171, 0x02c5, 0x0341, 0x015f, 0x03d8, 0x02ec, 0x035a,
    0x017d, 0x0380, 0x0041, 0x01e6, 0x007c, 0x03e0, 0x0376, 0x02df, 0x0048, 0x01ad, 0x0277, 0x0364, 0x003d, 0x027f, 0x0230, 0x00e5,
    0x03ea, 0x0032, 0x035f, 0x0261, 0x0319, 0x00e8, 0x0278, 0x0012, 0x030a, 0x0384, 0x0398, 0x0110, 0x0100, 0x03cc, 0x00a1, 0x03b8,
    0x01ff, 0x033c, 0x0365, 0x03ae, 0x03e1, 0x01f5, 0x02a3, 0x0291, 0x0366, 0x0014, 0x029e, 0x02e2, 0x03e9, 0x02f4, 0x0094, 0x0101,
    0x004b, 0x02e7, 0x0188, 0x0280, 0x0097, 0x0159, 0x00ea, 0x03b4, 0x015b, 0x02ce, 0x0140, 0x03b5, 0x007d, 0x0174, 0x0179, 0x0221,
    0x02a2, 0x03cd, 0x0022, 0x03c3, 0x0264, 0x03b6, 0x0367, 0x00f8, 0x009a, 0x0160, 0x0281, 0x0285, 0x017c, 0x030f, 0x0017, 0x00c0,
    0x0019, 0x00c4, 0x02c7, 0x00c5, 0x01bb, 0x03ce, 0x017e, 0x0024, 0x038a, 0x039e, 0x0282, 0x02c6, 0x00d2, 0x01bc, 0x0288, 0x02b7,
    0x02eb, 0x02ab, 0x0254, 0x02e0, 0x0235, 0x02db, 0x03eb, 0x033f, 0x0064, 0x03ec, 0x02cf, 0x03f0, 0x03ed, 0x01bf, 0x001a, 0x0245,
    0x00c7, 0x0106, 0x00d7, 0x03ee, 0x02b6, 0x0033, 0x0075, 0x009b, 0x0034, 0x026a, 0x0222, 0x01ee, 0x0310, 0x0192, 0x0147, 0x01ac,
    0x0266, 0x013c, 0x01b1, 0x019e, 0x0151, 0x017f, 0x026f, 0x0289, 0x0369, 0x0200, 0x0069, 0x00ce, 0x0232, 0x01f2, 0x03ef, 0x03b9,
    0x03f6, 0x032a, 0x0184, 0x026b, 0x0381, 0x026e, 0x0231, 0x001b, 0x01c2, 0x038d, 0x02e8, 0x00eb, 0x0042, 0x01da, 0x02f5, 0x0326,
    0x009d, 0x00ed, 0x021a, 0x00af, 0x01c0, 0x014e, 0x0161, 0x03fb, 0x022d, 0x0248, 0x001d, 0x02c9, 0x0383, 0x028a, 0x02ca, 0x0340,
    0x0309, 0x02a5, 0x0284, 0x0327, 0x001e, 0x026c, 0x02fb, 0x01ba, 0x0300, 0x0385, 0x039f, 0x0347, 0x0186, 0x0387, 0x00cf, 0x0344,
    0x0109, 0x0303, 0x0328, 0x00c9, 0x018a, 0x0066, 0x0301, 0x00ca, 0x0135, 0x0304, 0x028d, 0x038b, 0x02d7, 0x0270, 0x03f7, 0x030b,
    0x03bb, 0x0312, 0x01a6, 0x00a5, 0x0081, 0x001f, 0x0313, 0x00ee, 0x01f7, 0x02a9, 0x0314, 0x00cb, 0x038c, 0x036a, 0x0027, 0x0238,
    0x0345, 0x0111, 0x00d9, 0x0262, 0x02aa, 0x009e, 0x018b, 0x021b, 0x0107, 0x028c, 0x00a9, 0x00aa, 0x0315, 0x009f, 0x0316, 0x0126,
    0x038f, 0x0317, 0x00ef, 0x019c, 0x00f0, 0x0349, 0x02af, 0x00ab, 0x019f, 0x0141, 0x0025, 0x010b, 0x03bc, 0x0271, 0x0249, 0x0035,
    0x0318, 0x006a, 0x031c, 0x02b8, 0x0392, 0x03cf, 0x032b, 0x010c, 0x022c, 0x0393, 0x03a3, 0x03f8, 0x03a4, 0x03c5, 0x02b9, 0x0036,
    0x03a6, 0x031d, 0x018f, 0x00ae, 0x0122, 0x0037, 0x01e5, 0x003e, 0x006f, 0x0043, 0x0145, 0x03a5, 0x031f, 0x0190, 0x0070, 0x0138,
    0x03bd, 0x02ba, 0x00f2, 0x0130, 0x0146, 0x0044, 0x02bb, 0x0320, 0x012b, 0x0045, 0x0321, 0x03be, 0x03bf, 0x03c0, 0x03c1, 0x018c,
    0x0223, 0x0193, 0x01df, 0x00ac, 0x0198, 0x0203, 0x0199, 0x03c2, 0x00f3, 0x03c4, 0x019a, 0x0047, 0x03d0, 0x01ae, 0x004a, 0x019d,
    0x004d, 0x00ad, 0x004e, 0x004f, 0x0054, 0x0055, 0x01a1, 0x0071, 0x0077, 0x007e, 0x0112, 0x0113, 0x0114, 0x0134, 0x01b2, 0x0115,
    0x011d, 0x011e, 0x011f, 0x0123, 0x01a2, 0x01a4, 0x01b3, 0x012a, 0x0149, 0x012c, 0x0131, 0x0136, 0x014a, 0x014c, 0x01e7, 0x01b7,
    0x01c1, 0x01c3, 0x01c4, 0x01c5, 0x01c7, 0x01c9, 0x01d0, 0x01d2, 0x01d4, 0x01d6, 0x01d8, 0x01e1, 0x01e2, 0x01e8, 0x01e9, 0x01ea,
    0x01ec, 0x01ed, 0x01ef, 0x01f0, 0x01f8, 0x01f9, 0x0201, 0x01fc, 0x0224, 0x0226, 0x0227, 0x0228, 0x022a, 0x022b, 0x0236, 0x03ff
};

static unsigned char SPN[] = 
{
    0X05, 0X04, 0X00, 0X0C, 0X00, 0X01, 0X00, 0X08, 0X00, 0X00, 0X00, 0X00, 0XFF, 0XF7, 0X9E, 0X61, 0X00, 0X00, 0X00, 0X01, 
    0X00, 0X00, 0X00, 0X01, 0X00, 0X00, 0X00, 0X0B, 0X44, 0X45, 0X56, 0X4F, 0X4E, 0X49, 0X54, 0X2E, 0X43, 0X4F, 0X4D, 0X00, 
    0X00, 0X00, 0X03, 0X64, 0X73, 0X61, 0X00, 0X00, 0X00, 0X01, 0X00, 0X00, 0X00, 0X01, 0X00, 0X00, 0X00, 0X0B, 0X44, 0X45, 
    0X56, 0X4F, 0X4E, 0X49, 0X54, 0X2E, 0X43, 0X4F, 0X4D, 0X00, 0X00, 0X00, 0X03, 0X64, 0X73, 0X61, 0X00, 0X00, 0X00, 0X00, 
    0X00, 0X00, 0X00, 0X02, 0X00, 0X00, 0X00, 0X0B, 0X44, 0X45, 0X56, 0X4F, 0X4E, 0X49, 0X54, 0X2E, 0X43, 0X4F, 0X4D, 0X00, 
    0X00, 0X00, 0X06, 0X6B, 0X72, 0X62, 0X74, 0X67, 0X74, 0X00, 0X00, 0X00, 0X0B, 0X44, 0X45, 0X56, 0X4F, 0X4E, 0X49, 0X54, 
    0X2E, 0X43, 0X4F, 0X4D, 0X00, 0X17, 0X00, 0X00, 0X00, 0X10, 0X83, 0X01, 0XE7, 0XD7, 0X99, 0XF1, 0X06, 0XF6, 0XFF, 0X3F, 
    0X2E, 0X2A, 0X41, 0X4A, 0X66, 0X91, 0X47, 0X31, 0XE7, 0XC0, 0X47, 0X31, 0XE7, 0XBC, 0X47, 0X32, 0X74, 0X60, 0X47, 0X33, 
    0X39, 0X3C, 0X00, 0X40, 0XE0, 0X00, 0X00, 0X00, 0X00, 0X00, 0X00, 0X00, 0X00, 0X00, 0X00, 0X00, 0X00, 0X03, 0X72, 0X61, 
    0X82, 0X03, 0X6E, 0X30, 0X82, 0X03, 0X6A, 0XA0, 0X03, 0X02, 0X01, 0X05, 0XA1, 0X0D, 0X1B, 0X0B, 0X44, 0X45, 0X56, 0X4F, 
    0X4E, 0X49, 0X54, 0X2E, 0X43, 0X4F, 0X4D, 0XA2, 0X20, 0X30, 0X1E, 0XA0, 0X03, 0X02, 0X01, 0X00, 0XA1, 0X17, 0X30, 0X15, 
    0X1B, 0X06, 0X6B, 0X72, 0X62, 0X74, 0X67, 0X74, 0X1B, 0X0B, 0X44, 0X45, 0X56, 0X4F, 0X4E, 0X49, 0X54, 0X2E, 0X43, 0X4F, 
    0X4D, 0XA3, 0X82, 0X03, 0X30, 0X30, 0X82, 0X03, 0X2C, 0XA0, 0X03, 0X02, 0X01, 0X17, 0XA2, 0X82, 0X03, 0X23, 0X04, 0X82, 
    0X03, 0X1F, 0XC7, 0XE3, 0X3B, 0XA2, 0X38, 0X7A, 0XC9, 0X17, 0X87, 0XD8, 0XDE, 0X45, 0X77, 0XCE, 0XE9, 0X8B, 0XB6, 0X12, 
    0X8C, 0X61, 0XCB, 0XA9, 0X65, 0X44, 0X45, 0XFD, 0XA5, 0X5B, 0XB8, 0XC0, 0X39, 0X28, 0X9B, 0X97, 0X80, 0X52, 0X76, 0X5E, 
    0X6A, 0XB4, 0XD0, 0X3B, 0X19, 0X8F, 0X5A, 0XD4, 0X89, 0XD6, 0XEC, 0XB1, 0XC8, 0XB4, 0X2C, 0X30, 0XB7, 0XA9, 0XB0, 0X69, 
    0XE4, 0X2D, 0X67, 0X57, 0XD4, 0XD5, 0X41, 0X9B, 0X01, 0X5F, 0X34, 0X21, 0XA3, 0XA7, 0XCE, 0XF9, 0X6D, 0X38, 0X9B, 0X96, 
    0X86, 0X57, 0X27, 0XD6, 0XEF, 0X98, 0XC6, 0X09, 0X58, 0X9E, 0X3F, 0X82, 0XAF, 0X6C, 0X4B, 0X7E, 0X66, 0X04, 0X6A, 0XBE, 
    0XDE, 0X65, 0XEB, 0X41, 0X7D, 0XBA, 0XAF, 0XAA, 0X73, 0X5D, 0X66, 0X2C, 0X37, 0X34, 0X79, 0XFD, 0XFB, 0X37, 0X4B, 0X73, 
    0X54, 0XC3, 0XB1, 0X7A, 0X79, 0X88, 0X06, 0XC2, 0X79, 0XA8, 0XA3, 0XAC, 0X15, 0X66, 0X57, 0X49, 0XB0, 0X79, 0X67, 0X01, 
    0XAC, 0XF3, 0X82, 0X75, 0XC7, 0X46, 0X8E, 0XD6, 0X61, 0X2A, 0X2A, 0XFD, 0X30, 0XD9, 0X21, 0X32, 0XB1, 0X6C, 0X2A, 0X94, 
    0X96, 0XBF, 0X6D, 0X6C, 0XFF, 0X38, 0XA0, 0XFD, 0X5D, 0X74, 0XA7, 0X3E, 0XB7, 0XB7, 0X28, 0XD3, 0XF4, 0X81, 0XE8, 0X9F, 
    0XF2, 0X72, 0XE8, 0X9F, 0X8C, 0X2F, 0XAE, 0X7B, 0XD1, 0XE8, 0XAA, 0XBF, 0X92, 0XC7, 0X9C, 0X62, 0X56, 0X51, 0X3C, 0XC7, 
    0X11, 0X88, 0X52, 0X97, 0XDB, 0XAD, 0X58, 0X6E, 0X64, 0XB0, 0XF4, 0X25, 0XE7, 0XAD, 0X47, 0XCD, 0XCC, 0XD2, 0XC8, 0X68, 
    0XF6, 0X21, 0X0F, 0X18, 0XB4, 0XAF, 0X95, 0X3E, 0XAF, 0XFC, 0XB7, 0X6C, 0X25, 0X03, 0X1E, 0X5E, 0X48, 0X3A, 0XFD, 0XBD, 
    0X1E, 0X7A, 0XCD, 0X77, 0X98, 0XE9, 0X76, 0X0B, 0XC6, 0X87, 0XB2, 0XC6, 0XA9, 0X09, 0X01, 0X6E, 0X9B, 0X89, 0X76, 0XCE, 
    0XE8, 0XDE, 0XD5, 0X29, 0X33, 0X26, 0X49, 0XA8, 0XB5, 0XA2, 0XF5, 0XE2, 0XE9, 0X0F, 0X0B, 0X16, 0X7F, 0X2B, 0X5C, 0X96, 
    0XE2, 0XFF, 0X82, 0X0C, 0X16, 0XAE, 0XC1, 0X9A, 0XC1, 0X2D, 0XB3, 0X0C, 0X1C, 0XA0, 0XA8, 0X82, 0X7A, 0XA1, 0X7C, 0X60, 
    0XB4, 0XA0, 0X9A, 0XAE, 0X4B, 0XB8, 0XF0, 0X02, 0X03, 0XF6, 0XA8, 0X32, 0X44, 0X61, 0XAC, 0X2D, 0X3C, 0XFF, 0X1E, 0X8B, 
    0X97, 0X6B, 0X01, 0XB8, 0XDB, 0XD5, 0X2C, 0X3A, 0X7C, 0X22, 0XEA, 0X51, 0X0B, 0XB5, 0X8F, 0XDC, 0XC1, 0X42, 0X71, 0XAC, 
    0X62, 0X56, 0XE2, 0X3C, 0XF4, 0XE3, 0XE1, 0X47, 0XDE, 0X67, 0X30, 0X23, 0X8B, 0X51, 0X6F, 0X0C, 0X20, 0XF0, 0X6D, 0X78, 
    0X47, 0X78, 0XA3, 0XB1, 0XE8, 0X27, 0XD2, 0X17, 0X45, 0X41, 0XB1, 0X25, 0XBA, 0X21, 0X35, 0X35, 0X46, 0XA4, 0XF1, 0XB7, 
    0XE4, 0XC9, 0X8D, 0X75, 0X33, 0X5E, 0XCF, 0X3B, 0XDA, 0X2C, 0XF4, 0X19, 0X57, 0XC7, 0XCA, 0X6B, 0X7E, 0XB2, 0X21, 0XF0, 
    0XA9, 0X2C, 0XB4, 0X79, 0XD6, 0X74, 0X48, 0X80, 0XED, 0XFE, 0XCC, 0X9B, 0X9D, 0XB1, 0XC5, 0X16, 0X5A, 0XA3, 0X29, 0X0D, 
    0X8C, 0X93, 0X1F, 0X45, 0XD6, 0XEF, 0XA4, 0X46, 0X2F, 0XE9, 0X3B, 0XD8, 0X97, 0XCB, 0XE8, 0XE1, 0XCE, 0X57, 0XF7, 0XDE, 
    0XC9, 0X90, 0XFA, 0XAF, 0X51, 0XEA, 0X61, 0XBB, 0XC8, 0XBB, 0X6E, 0XA6, 0XD9, 0X63, 0X03, 0XCF, 0X9D, 0XD7, 0X07, 0XCF, 
    0XA1, 0XC5, 0X30, 0X10, 0X29, 0X6B, 0X2C, 0XD8, 0XF4, 0X9D, 0X8E, 0XEB, 0X25, 0X8D, 0X3A, 0X8B, 0X7F, 0X4D, 0X75, 0X2C, 
    0X91, 0X50, 0X28, 0X8E, 0XD4, 0X4B, 0X07, 0XC3, 0X6E, 0X42, 0X54, 0XB7, 0XE4, 0X8E, 0XF4, 0X52, 0X47, 0X73, 0XF0, 0XDA, 
    0XCB, 0XCF, 0X80, 0X88, 0X09, 0XBF, 0XD9, 0X4D, 0XA4, 0X7E, 0X5B, 0X50, 0X59, 0XDB, 0X50, 0X66, 0X03, 0X0D, 0XAD, 0XD9, 
    0X5B, 0X28, 0XAC, 0X7E, 0X31, 0X4B, 0X7E, 0XD0, 0XE1, 0X2B, 0X30, 0X45, 0X22, 0X08, 0XB2, 0X69, 0X79, 0XDD, 0XB4, 0X72, 
    0XE4, 0X26, 0XF3, 0X31, 0X84, 0XE2, 0XB5, 0XAD, 0X2D, 0X1B, 0XD8, 0X9C, 0X4E, 0X85, 0XC7, 0X5F, 0X9F, 0X1F, 0XB7, 0XDA, 
    0X54, 0XFF, 0XC8, 0XEB, 0XF0, 0X87, 0X51, 0X2C, 0X42, 0X9D, 0XFE, 0X53, 0X95, 0X78, 0XBB, 0X3E, 0XBF, 0X11, 0XAB, 0X5C, 
    0X69, 0XB9, 0X11, 0X54, 0X68, 0XB6, 0X1A, 0X18, 0X30, 0X94, 0X99, 0X9B, 0X7B, 0X9A, 0X84, 0X43, 0X22, 0X49, 0XDF, 0XAF, 
    0X9D, 0X17, 0XA9, 0X62, 0X69, 0X7A, 0XD7, 0XF2, 0XF9, 0X1A, 0X3E, 0X1A, 0XB2, 0X46, 0XF1, 0XD9, 0XDA, 0X05, 0XD7, 0X1A, 
    0X1D, 0X65, 0XE8, 0X7D, 0X2C, 0X94, 0X67, 0XAE, 0X05, 0XC4, 0XC3, 0X01, 0X78, 0XB7, 0XEA, 0XCC, 0XBF, 0X80, 0XC3, 0X3F, 
    0XAA, 0X1A, 0XB5, 0X0F, 0X42, 0XD7, 0X7F, 0X4A, 0XDF, 0XF2, 0XB8, 0X41, 0XA8, 0X9E, 0X3E, 0XE1, 0X57, 0X79, 0X7C, 0X50, 
    0X91, 0X4D, 0XAE, 0XC3, 0X98, 0X49, 0X2A, 0X34, 0XFC, 0XBB, 0X81, 0XDF, 0X89, 0X09, 0X9F, 0X48, 0X99, 0X26, 0X87, 0X97, 
    0XDE, 0X04, 0X7B, 0XDD, 0X78, 0XB0, 0XE2, 0X02, 0XCA, 0X19, 0X9A, 0X83, 0X57, 0XCC, 0X31, 0X3D, 0X26, 0X0C, 0X69, 0X1E, 
    0X17, 0XAD, 0XAC, 0X10, 0XAA, 0XBB, 0XA7, 0X61, 0X08, 0X4A, 0X64, 0XE3, 0X40, 0XFD, 0X50, 0X0A, 0XC3, 0X14, 0XA2, 0XBE, 
    0XDF, 0X52, 0X27, 0XD2, 0X33, 0X19, 0X2E, 0X72, 0X83, 0XC9, 0X71, 0X3B, 0X43, 0X15, 0X71, 0X08, 0XDE, 0XF6, 0X6F, 0X10, 
    0X59, 0X0F, 0X53, 0X91, 0X01, 0X84, 0X40, 0X5D, 0XD2, 0X66, 0X46, 0X21, 0XF1, 0X4C, 0X64, 0XBC, 0XBE, 0X68, 0X11, 0X90, 
    0X15, 0X04, 0X9B, 0X34, 0X9B, 0X93, 0X20, 0X6C, 0XAC, 0XE7, 0X9D, 0X5B, 0X28, 0XF0, 0XE3, 0X39, 0X6D, 0X70, 0XF6, 0X09, 
    0X42, 0X67, 0X91, 0X92, 0XCB, 0X0D, 0X1B, 0X4D, 0XDD, 0X0D, 0XAA, 0X99, 0XA2, 0XB5, 0X41, 0X2F, 0X5F, 0XF3, 0XE0, 0X40, 
    0X2A, 0X00, 0X00, 0X00, 0X00
};

#ifdef __cplusplus
extern "C"
#endif
char*
create_tgt_response(char* filename)
{
    FILE* f = fopen(filename, "rb");
    if (f == NULL)
        return 0;

    fseek(f, 0, SEEK_END);
    int l = ftell(f);
    fseek(f, 0, SEEK_SET);

    unsigned char* buf = (unsigned char*)malloc(16 + l);
    *(unsigned int*)buf = 1;
    *(unsigned int*)(buf + 4) = 0;
    *(unsigned int*)(buf + 8) = sizeof(SPN);
    *(unsigned int*)(buf + 12) = l;

    memcpy(buf + 16, SPN, sizeof(SPN));
    fread(buf + 16 + sizeof(SPN), 1, l, f);

    fclose(f);

    l += 16 + sizeof(SPN);
    char* res = (char*)malloc(l * 2 + 1);
    BinToHex(buf, l, res, l * 2 + 1);
    free(buf);
    return res;
}

static unsigned int Checksum(unsigned char* pBuffer, int cbBuffer)
{
    unsigned int dwChecksum = 0;
    
    for (int i = 0; i < cbBuffer; i++)
    {
        dwChecksum += pBuffer[i];
    }
    
    return dwChecksum % 256;
}

void Encode256(unsigned char* rgbBuffer, char* szString)
{
    int i;
    
    // Randomize the entire buffer.
    srand(time(0));
    for (i = 0; i < 256; i++)
    {
        rgbBuffer[i] = rand() % 256;
    }
    
    // Scatter the password in the buffer.
    for (i = 0; i < 256; i++)
    {
        rgbBuffer[scatter256[i]] = (unsigned char)szString[i];
        if (szString[i] == '\0') break;
    }
    
    // Force the checksum for the entire buffer equal to zero.
    rgbBuffer[255] = (unsigned char)(256 - Checksum(rgbBuffer, 255));
}

void Decode256(unsigned char *rgbBuffer, char *szString)
{
    for (int i = 0; i < 256; i++)
    {
    	szString[i] = (unsigned char) rgbBuffer[scatter256[i]];
    	if (szString[i] == '\0')
    	{
    		break;
    	}
    }
}

void Encode1024(unsigned char* rgbBuffer, char* szString)
{
    int i;
    
    // Randomize the entire buffer.
    srand(time(0));
    for (i = 0; i < 1024; i++)
    {
        rgbBuffer[i] = rand() % 256;
    }
    
    // Scatter the password in the buffer.
    for (i = 0; i < 1024; i++)
    {
        rgbBuffer[scatter1024[i]] = (unsigned char)szString[i];
        if (szString[i] == '\0') break;
    }
    
    // Force the checksum for the entire buffer equal to zero.
    rgbBuffer[1023] = (unsigned char)(256 - Checksum(rgbBuffer, 1023));
}

void Decode1024(unsigned char *rgbBuffer, char *szString)
{
    for (int i = 0; i < 1024; i++)
    {
    	szString[i] = (unsigned char) rgbBuffer[scatter1024[i]];
    	if (szString[i] == '\0')
    	{
    		break;
    	}
    }
}

int BinToHex(unsigned char* pBin, int cbBin, char* pHex, int cbHex)
{
    // Initialize the output buffer.
    memset(pHex, 0, cbHex);
    
    // Check the size of the output buffer.
    if (cbHex < ((cbBin * 2) + 1)) 
        return 0;
    
    // Convert each byte to its hexadecimal equivalent.
    for (int i = 0; i < cbBin; i++)
    {
        pHex += sprintf(pHex, "%02X", pBin[i]);
    }
    *pHex = '\0';
    
    return cbBin;
}

int HexToBin(unsigned char* pBin, int cbBin, char* pHex, int cbHex)
{
    int i;
    // Initialize the output buffer.
    memset(pBin, 0, cbBin);
    
    // Check the size of the output buffer.
    if (cbBin < ((cbHex / 2) + 1)) 
    {
        return 0;
    }

	for (i = 0; i < cbHex/2; ++i)
	{
		int tens = pHex[i*2] > 64 ? pHex[i*2] - 55 : pHex[i*2] - 48;
		int ones = pHex[i*2+1] > 64 ? pHex[i*2+1] - 55 : pHex[i*2+1] - 48;
		pBin[i] = tens*16 + ones;
	}
	pBin[i] = '\0';
    return cbHex;
}

void Encrypt(unsigned char* pBuffer, int cbBuffer)
{
    unsigned char key[] = { 0x84, 0x31, 0xA7, 0xF3, 0x4E, 0x2B, 0x90, 0x59 };
    RC4StreamCipher cipher(key, sizeof(key));
    cipher.Encrypt(pBuffer, pBuffer, cbBuffer);
}

void Decrypt(unsigned char* pBuffer, int cbBuffer)
{
    unsigned char key[] = { 0x84, 0x31, 0xA7, 0xF3, 0x4E, 0x2B, 0x90, 0x59 };
    RC4StreamCipher cipher(key, sizeof(key));
    cipher.Decrypt(pBuffer, pBuffer, cbBuffer);
}

