
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef _FARM_H_
#define _FARM_H_

#include <QSettings>
#include <QTemporaryFile>
#include <QTextStream>
#include <QList>
#include <QMessageBox>
#include <QFile>
#include <QDomNode>
#include <QByteArray>

#include "location.h"
#include "FarmClient/applicationList.h"

class Farm
{
public:
        Farm(QSettings *s, QString key = QString(),
             const QString & name = QString());
	virtual ~Farm();

	QFile * createPitFile();
	bool fromBuffer(const QByteArray & buffer);

	bool loadFarm();
	void saveFarm();

        QString getKey() const { return key; }

        // Location management
        Location * createNewLocation();
        int addLocation(Location *location);
        void removeLocation(int number);

	void setCurrentLocation(int number) { currentLocation = locations.at(number); }
	Location * getCurrentLocation() { return currentLocation; }
	Location * getLocation(int number) { return locations.at(number); }
	QStringList getLocationNamesList();

	void setLastLocation(int number) { iLast = number; }
	int getLastLocation() { return iLast; }
	void setDefaultLocation(int number) { iDefault = number; }
        int getDefaultLocation() { return iDefault; }
        bool isPromptLocation() { return promptLocation; }
        void setPromptLocation(bool prompt) { promptLocation = prompt; }

        QString getName() const;
        void setName(const QString & n);
        bool verifyName(const QString & name);

        QString  getUsername() const { return username; }
        void setUsername(const QString & name) { username = name; }
        QString  getPassword() const { return password; }
	void setPassword(const QString & pass) { password = pass; }
        QStringList  getDomainList() const { return domainList; }
        void setDomainList(const QStringList & dmns) { domainList = dmns; }
        QString getCurrentDomain() const { return currentDomain; }
        void setCurrentDomain(const QString & dmn) { currentDomain = dmn; }
        bool isCredentialsSaved() const { return saveCredentials; }
	void setCredentialsSave(bool save) { saveCredentials = save; }
        bool isKerberosMode() const { return kerberosMode; }
	void setKerberosMode(bool km) { kerberosMode = km; }
        bool isKerberosEnabled() const { return kerberosEnabled; }
        void setKerberosEnable(bool mode) { kerberosEnabled = mode; }
        bool isSsoEnabled() const { return ssoEnabled; }
        void setSsoEnable(bool mode) { ssoEnabled = mode; }
        int getWaitTime() const { return waitTime; }
        void setWaitTime(int wait) { waitTime = wait; }
        int getNumberOfRetries() const { return numberOfRetries; }
        void setNumberOfRetries(int retry) { numberOfRetries = retry; }
        bool waitForDesktop() const { return waitForDesktopIsEnabled; }
        void setWaitForDesktop( bool enabled ) { waitForDesktopIsEnabled = enabled; }

	// User experience
        bool isWallpaperEnabled() const { return wallpaperEnabled; }
	void setWallpaperEnable(bool enabled) { wallpaperEnabled = enabled; }
        bool isDraggingEnabled() const { return draggingEnabled; }
	void setDraggingEnable(bool enabled) { draggingEnabled = enabled; }
        bool isAnimationEnabled() const { return animationEnabled; }
	void setAnimationEnable(bool enabled) { animationEnabled = enabled; }
        bool isThemesEnabled() const { return themesEnabled; }
	void setThemesEnable(bool enabled) { themesEnabled = enabled; }
        bool isBitmapCachingEnabled() const { return bitmapCachingEnabled; }
        void setBitmapCachingEnable(bool enabled) { bitmapCachingEnabled = enabled; }

        bool isMmrEnabled() const { return mmrEnabled; }
        bool isFmmrEnabled() const { return fmmrEnabled; }
	bool isGaEnabled() const { return gaEnabled; }
        void setMmrEnable(bool enabled) { mmrEnabled = enabled; }
        void setFmmrEnable(bool enabled) { fmmrEnabled = enabled; }
        void setGaEnable(bool enabled) { gaEnabled = enabled; }

        bool isReconnection() const { return reconnect; }
        void setReconnection(bool b) { reconnect = b; }

	// Desktop options
        int getDesktopWidth() const { return desktopWidth; }
	void setDesktopWidth(int width) { desktopWidth = width; }
        int getDesktopHeight() const { return desktopHeight; }
	void setDesktopHeight(int height) { desktopHeight = height; }
        int getColorDepth() const { return colorDepth; }
	void setColorDepth(int depth) { colorDepth = depth; }

        bool isConnectionBar() const { return connectionBar; }
        void setConnectionBar(bool mode) { connectionBar = mode; }
        bool isPinConnectionBar() const { return pinConnectionBar; }
        void setPinConnectionBar(bool mode) { pinConnectionBar = mode; }
        bool isSpanMonitors() const { return spanMonitors; }
        void setSpanMonitors(bool mode) { spanMonitors = mode; }
        bool isSmartSizing() const { return smartSizing; }
        void setSmartSizing(bool mode) { smartSizing = mode; }
        bool isSeamlessMode() const { return seamlessMode; }
	void setSeamlessMode(bool smode) { seamlessMode = smode; }

	// Redirection options
        bool isRedirectClipboard() const { return redirectClipboard; }
	void setRedirectClipboard(bool rc) { redirectClipboard = rc; }
        bool isRedirectDrives() const { return redirectDrives; }
	void setRedirectDrives(bool rd) { redirectDrives = rd; }
        bool isRedirectPrinters() const { return redirectPrinters; }
	void setRedirectPrinters(bool rp) { redirectPrinters = rp; }
        bool isRedirectUPrinters() const { return redirectUPrinters; }
	void setRedirectUPrinters(bool rp) { redirectUPrinters = rp; }
        bool isRedirectComPorts() const { return redirectComPorts; }
	void setRedirectComPorts(bool rc) { redirectComPorts = rc; }
        bool isRedirectHandhelds() const { return redirectHandhelds; }
	void setRedirectHandhelds(bool rh) { redirectHandhelds = rh; }
        bool isRedirectMicrophone() const { return redirectMicrophone; }
	void setRedirectMicrophone(bool rm) { redirectMicrophone = rm; }
        bool isRedirectSmartCards() const { return redirectSmartCards; }
	void setRedirectSmartCards(bool rs) { redirectSmartCards = rs; }

	// Sound mode
        QString getSoundMode() const { return soundMode; }
	void setSoundMode(const QString & mode) { soundMode = mode; }

	// Downstream audio
	bool isDownstreamAudioEnabled() const { return downstreamAudioEnabled; }
	void setDownstreamAudioEnabled(bool daEnabled) { downstreamAudioEnabled = daEnabled; }
	QString getDAMode() const { return daMode; }
	void setDAMode(const QString & mode) { daMode = mode; }
	QString getDAQuality() const { return daQuality; }
	void setDAQuality(const QString & quality) { daQuality = quality; }
	int getDABuffering() const { return daBuffering; }
	void setDABuffering(const int buf) { daBuffering = buf; }
	int getDACustomQuality() const { return daCustomQuality; }
	void setDACustomQuality(const int cq) { daCustomQuality = cq; }

	// Upstream audio
	bool isUpstreamAudioEnabled() const { return upstreamAudioEnabled; }
	void setUpstreamAudioEnabled(bool uaEnabled) { upstreamAudioEnabled = uaEnabled; }
	QString getUAMode() const { return uaMode; }
	void setUAMode(const QString & mode) { uaMode = mode; }
	int getUAQuality() const { return uaQuality; }
	void setUAQuality(const int quality) { uaQuality = quality; }
	int getUAComplexity() const { return uaComplexity; }
	void setUAComplexity(const int complexity) { uaComplexity = complexity; }
	bool getUAEnhancedCompression() const { return uaEnhancedCompression; }
	void setUAEnhancedCompression(bool ec) { uaEnhancedCompression = ec; }

	// Settings hide
        bool isHideSettings() const { return hideSettings; }
	void setHideSettings(bool v) { hideSettings = v; }

	// Password management
        QString getPasswordManagementServer() const { return pmServer; }
	void setPasswordManagementServer(QString server) { pmServer = server; }
        int getPasswordManagementPort() const { return pmPort; }
	void setPasswordManagementPort(int port) { pmPort = port; }
        bool isPasswordManagementAllowed() const { return allowPasswordManagement; }
        void setPasswordManagementAllow(bool allow) { allowPasswordManagement = allow; }

        // Load farm from server parameters
        QString getSourceProtocol() const { return sourceProtocol; }
        void setSourceProtocol(const QString & protocol) { sourceProtocol = protocol; }
        QString getSourceServer() const { return sourceServer; }
        void setSourceServer(const QString & server) { sourceServer = server; }
        QString getSourceFile() const { return sourceFile; }
        void setSourceFile(const QString & file) { sourceFile = file; }
        QString getSourceProxy() const { return sourceProxy; }
        void setSourceProxy(const QString & proxy) { sourceProxy = proxy; }

    // Applications management
    ApplicationList applications() const;
    void setApplications(ApplicationList apps);
    ApplicationList autolaunch() const;
    void setAutolaunch(ApplicationList apps);
    //QStringList getAutoLaunchList() const { return autoLaunch; }
    //void setAutoLaunchList(QStringList apps) { autoLaunch = apps; }

    void setNeedsReconnecting(bool need) { needReconnect = need; }
    bool needsReconnecting() const { return needReconnect; }

    void setAutoConnect(bool val) { autoConnect = val; }
    bool autoConnected() const { return autoConnect; }


private:
	void parseNextNode(const QDomNode & node);

	QString name;
	QString username;
	QString password;
	QStringList domainList;
	QString currentDomain;
	bool kerberosMode;
        bool kerberosEnabled;
        bool ssoEnabled;
	bool saveCredentials;

	bool wallpaperEnabled;
	bool draggingEnabled;
	bool animationEnabled;
	bool themesEnabled;
        bool bitmapCachingEnabled;
        bool mmrEnabled;
        bool fmmrEnabled;
	bool gaEnabled;
        bool reconnect;

	int desktopWidth;
	int desktopHeight;
	int colorDepth;
        bool connectionBar;
        bool pinConnectionBar;
        bool spanMonitors;
        bool smartSizing;
	bool seamlessMode;

        QString soundMode;

    bool downstreamAudioEnabled;
    QString daMode;
    QString daQuality;
    int daBuffering;
    int daCustomQuality;

    bool upstreamAudioEnabled;
    QString uaMode;
    int uaQuality;
    int uaComplexity;
    bool uaEnhancedCompression;

	bool redirectClipboard;
	bool redirectDrives;
	bool redirectPrinters;
	bool redirectUPrinters;
	bool redirectComPorts;
	bool redirectHandhelds;
	bool redirectMicrophone;
	bool redirectSmartCards;

	bool hideSettings;
	QString pmServer;
	int pmPort;
        bool allowPasswordManagement;

        QString sourceServer;
        QString sourceProtocol;
        QString sourceFile;
        QString sourceProxy;

	int iLast;
        int iDefault;
        bool promptLocation;

    bool needReconnect;
    bool autoConnect;

    int waitTime;
    int numberOfRetries;
    bool waitForDesktopIsEnabled;

	QString key;

	QSettings *settings;
	QList<Location *> locations;
        Location *currentLocation;

    ApplicationList p_applications;
    ApplicationList p_autolaunch;
    QStringList autoLaunch;
};

#endif
