
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef MAIN_WINDOW_HPP
#define MAIN_WINDOW_HPP

#include <QMainWindow>
#include <QListWidget>
#include <QProgressBar>
#include <QAction>
#include <QMenu>
#include <QMenuBar>
#include <QApplication>
#include <QSettings>
#include <QTemporaryFile>
#include <QToolButton>
#include <QMutex>
#include "treeMenuWidget.h"
#include "verticalMenu.h"
#include "aboutDialog.h"

#include "FarmClient/farmConnection.h"
#include "ProgressDialog/progressDialog.h"

class QCheckBox;

class MainWindow : public QMainWindow
{
  Q_OBJECT

public:
    static MainWindow* instance()
    {
      static QMutex mutex;
      if (!m_instance)
      {
        mutex.lock();

        if (!m_instance)
          m_instance = new MainWindow();

        mutex.unlock();
      }

      return m_instance;
    }

protected:
    virtual void contextMenuEvent(QContextMenuEvent *pe);
    virtual void closeEvent(QCloseEvent * event);

private:
    MainWindow(QWidget *parent = 0);
    MainWindow(const MainWindow& );

private slots:
    void changeCurrentConnection();
    void logonAsADifferentUser();
    void changePassword();
    void refreshApplicationSet();
    void menuBarSlot();
    void toolBarSlot();
    void enableCredentialsPassThrough();
    void about();

    // Connection slots
    void connectionFinished(FarmConnection::FinishCode exitCode);
    void connectionFinishedAlt(FarmConnection::FinishCode exitCode, FarmConnection *connection);
    void connectionError();
    void runApplication(const QString &);
    void showStatusMessage(const QString &);

    // Farm menu
    void editFarm();
    void runFarm(QListWidgetItem *item = 0);
    void autoConnectedFarmChanged(int);

private:
    void createMenus();
    void createActions();
    void createToolBars();
    void createVerticalMenus();
    void createStatusBar();

    void setIcons();
    void writeSettings();
    void readSettings();

    void setStatus(const QString &);

    // Farm management
    void createFarmMenu();
    void updateFarmsList();
    QVBoxLayout *farmMenuGridLayout;
    QListWidget *farmListWidget;

    TreeMenuWidget *treeMenuWidget;
    QMenu *actionMenu;
    QMenu *settingsMenu;
    QMenu *helpMenu;
    QMenu *trayIconMenu;
    QToolBar *mainToolBar;
    VerticalMenu *farmVerticalMenu;
    VerticalMenu *actionVerticalMenu;

    QAction *manageConnectionAction;
    QAction *changeCurrentConnectionAction;
    QAction *logonAsADifferentUserAction;
    QAction *changePasswordAction;
    QAction *refreshApplicationSetAction;
    QAction *closeAction;

    //QAction *displaySettingsAction;
    QAction *menuBarAction;
    QAction *toolBarAction;
    QAction *enableCredentialsPassThroughAction;
    QAction *helpAction;
    QAction *aboutAction;

    QVBoxLayout *mainLayout;

    ProgressDialog *progressDialog;

    FarmSettings settings;

    FarmConnection *currentConnection;

    static MainWindow *m_instance;

};


#endif

