
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


/*
 *  tusbd/fifo.h
 *
 *  Copyright (C) 2007-2008 IncentivesPro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef _FIFO_H
#define _FIFO_H

struct fifo_descriptor
{
	struct list_head entrylist;	// list of entries
	spinlock_t lock;
	int num_entries;			// number of entries in fifo /*TODO: atomic operations*/
	wait_queue_head_t waitqueue;
};

struct fifo_entry
{
	struct list_head entry;
	int size;		// size in bytes of the event NOT including terminating zero
	int read;		// number of bytes read
	char ispointer:1;
	char isusermodepointer:1;
	union
	{
		u8 *pdata;
		u8 data[1];
	};
};

struct fifo_descriptor *usbd_fifo_create(void);
void usbd_fifo_destroy(struct fifo_descriptor *plogdesc);

void usbd_fifo_clear(struct fifo_descriptor *plogdesc);
int usbd_fifo_add(struct fifo_descriptor *pfifodesc, const void *buf, int bufsize, int user_mode_buffer);
// usbd_fifo_add_pointer does not copy the buffer, it queues a pointer to the buffer
int usbd_fifo_add_pointer(struct fifo_descriptor *pfifodesc, const void *buf, int bufsize, int user_mode_buffer);
int usbd_fifo_get(struct fifo_descriptor *plogdesc, void *buf, int bufsize, int user_mode_buffer);

#endif // _FIFO_H
