#!/bin/sh

if [ "$(id -u)" != "0" ]
then
  echo "Must be run with superuser privileges."
  echo
  exit 1
fi

# include and run the prerequisites checking scripts
. ./prereqchecker.sh $1

if [ "$1" = "-force-build" ]
then
echo
echo 'Force build mode is enabled - will try to build all modules'
echo
enable_usb=1
enable_mmr=1
enable_fmmr=1
enable_mic=1
build_gui=1

else
#
# Check to see if any of the components cannot be built
#
echo ''
echo 'Summary of available components:'
echo ''
if [ $enable_usb -eq 1 ]
then
    echo "USB Redirection support will be installed."
else
    echo "USB Redirection Support prequisites are not met. This component will not be installed."
fi

if [ $enable_mmr -eq 1 ]
then
    echo "MMR Support will be installed."
else
    echo "MMR Support prerequisites are not met.  This component will not be installed."
fi

if [ $enable_fmmr -eq 1 ]
then
    echo "Flash MMR Support will be installed."
else
    echo "Flash MMR Support prerequisites are not met.  This component will not be installed."
fi

if [ $enable_mic -eq 1 ]
then
    echo "Microphone Redirection will be installed."
else
    echo "Microphone Redirection prequisites are not met.  This component will not be installed."
fi

if [ $build_gui -eq 1 ]
then 
    echo "The Connection Manager will be built and installed."
else
    echo "The Connection Manager will not be built or installed."
fi

echo
echo -n 'Do you wish to continue the build and installation process? [Y/N]'
read response

if [ \( "$response" = "Y" \)  -o  \( "$response" = "y" \) ]
then  
    echo
    echo "The installation will now begin."
    echo
else
    exit 0
fi
fi

if [ $build_gui -eq 1 ]
then
echo
echo '-------------------------------------------------------'
echo 'Building Connection Manager'
echo '-------------------------------------------------------'

# Build the connmanager executable
cd ./manager
echo
echo "Performing make"
echo
#sleep 1
rm -f connmanager

# Prep the files that need to be writable
chmod +w *akefile*

$QT_QMAKE

make clean
make

# Check to make sure that the executable actually built.
if [ -f connmanager ]
then
cp connmanager /usr/local/bin
echo
echo 'Make completed'
sleep 1
echo
else
echo
echo 'Failed to build connection manager. Stopping here.'
echo
if [ "$1" != "-force-build" ]
then
  exit 1
fi
fi
cd ..
fi

echo
echo '-------------------------------------------------------'
echo 'Executing pntsc build'
echo '-------------------------------------------------------'
echo

# Install the MMR client plugin
cd pntsc
#sleep 2
rm -f pntsc
make clean
make

# Check to make sure that the executable actually built.
if [ -f pntsc ]
then
echo
echo 'pntsc build completed sucessfully.'
echo 'Installing pntsc now.'
make install
echo
else
echo
echo 'Failed to build pntsc. Stopping here.'
echo
if [ "$1" != "-force-build" ]
then
exit 1
fi
fi

echo '--------------------------------------------------------'
echo 'Building and installing qrdesktop with smardcard support'
echo '--------------------------------------------------------'
echo

# Build rdesktop
cd ../rdesktop

# Generate the Makefile
autoconf
rm -f Makefile

# Prepare any file that were unwritable
chmod +w config*

/bin/sh configure --enable-smartcard --with-mm-support

# Check to make sure that the executable actually built.
if [ -f Makefile ]
then
echo
echo 'Makefile was successfully generated.'
echo 'Building qrdesktop now.'
echo
sleep 1
else
echo
echo 'Failed to generate Makefile. Installation will abort here.'
echo
if [ "$1" != "-force-build" ]
then
exit 1
fi
fi

# Generate the rdesktop
rm -f qrdesktop
make clean

# make an alias for boost-thread library
if [ ! -f /usr/lib/libboost_thread.so ]
then
  ln /usr/lib/libboost_thread-mt.so /usr/lib/libboost_thread.so
fi
if [ ! -f /usr/lib/libmpg123.so.0 ]
then
  ln -s /usr/local/lib/libmpg123.so.0 /usr/lib/libmpg123.so.0
fi

# now build
make

# Check to make sure that the executable actually built.
if [ -f qrdesktop ]
then
echo
echo 'qrdesktop was sucessfully built.'
echo 'Installing qrdesktop.'
sleep 1
make install

# Assuming that the install script worked as intended, set
# the permissions on the qrdesktop executable to run as root
# this should be fairly safe and certainly no more unsafe than
# much of what is already being done.
chmod +s /usr/local/bin/qrdesktop


# for Fedora make a fix to the libmpg123 to prevent SELinux alarm
if [ -f /etc/redhat-release ]; then
  chcon -t textrel_shlib_t /usr/lib/libmpg123.so.0
fi
echo
else
echo
echo 'Failed to build qrdesktop. Stopping here.'
echo
if [ "$1" != "-force-build" ]
then
exit 1
fi
fi

echo '-------------------------------------------------------'
echo 'Building and installing session server'
echo '-------------------------------------------------------'
echo

# Build server
cd ../server
#sleep 2

# Generate the Makefile
rm -f pncm
make clean
make

# Check to make sure that the executable actually built.
if [ -f pncm ]
then
echo
echo 'Built session server(pncm) successfully'
echo 'Installing.'
#sleep 1
make install
echo 
else
echo
echo 'Failed to build session server (pncm). Stopping here.'
echo
if [ "$1" != "-force-build" ]
then
exit 1
fi
fi


echo '-------------------------------------------------------'
echo 'Installing extensions'
echo '-------------------------------------------------------'
echo

cd ../extensions
sleep 1

if [ $enable_mmr -eq 1 ]
then
echo '-------------------------------------------------------'
echo 'Installing MMR client'
echo '-------------------------------------------------------'
echo

cd mmr

cd client
sleep 1
make install

cd plugin
/bin/sh plugin_install
cd ../../..

echo
echo '-------------------------------------------------------'
echo 'Upgrading GStreamer ximagesink'
echo '-------------------------------------------------------'
echo

GST_PLUGINS_DIR=`pkg-config --variable=pluginsdir $GST_MAIN_MODULE`
if [ -f $GST_PLUGINS_DIR/libgstximagesink.so ]
then
cd mmr/ximagesink
make install
if [ -f /etc/redhat-release ]; then
make SE-fix
fi
echo
echo 'ximagesink upgrade completed.'
echo
else
echo
echo 'Cannot find an original GStreamer-0.10 ximagesink plugin to upgrade. Stopping here.'
echo
if [ "$1" != "-force-build" ]
then
exit 1
fi
fi
cd ../..
fi

if [ $enable_fmmr -eq 1 ]
then
echo
echo '-------------------------------------------------------'
echo 'Installing Flash MMR client'
echo '-------------------------------------------------------'
echo
cd fmmr
chmod +x fmmr
cp -f fmmr /usr/local/bin
echo
echo ' done.'
echo
cd ..
fi

if [ $enable_usb -eq 1 ]
then
echo
echo '-------------------------------------------------------'
echo 'Installing USB redirection server'
echo '-------------------------------------------------------'
echo

# make a link named /usr/src/linux
ln -sf $kernel_sources_dir/$kernel_sources_dir_prefix /usr/src/linux

cd usb-server
/bin/sh installer.sh install
cd ../..
fi

echo
echo '-------------------------------------------------------'
echo 'Installing logging'
echo '-------------------------------------------------------'
echo

QUESTCONF='/etc/rsyslog.d/quest.conf'
RSYSLOGD_PATH='/etc/rsyslog.d'

if [ ! -s $QUESTCONF ]
then
  echo 'Logging not found. Start install.'
  WHERE='/etc/rsyslog.d/quest.conf'

  if [ ! -d RSYSLOGD_PATH ]
  then
    WHERE='/etc/rsyslog.conf'		
  fi

  echo ':app-name,contains,"pnserver"	-/var/log/quest/pnserver.log'  >> $WHERE
  echo ':app-name,contains,"pntsc"	-/var/log/quest/pntsc.log'     >> $WHERE
  echo ':app-name,contains,"qrdesktop"	-/var/log/quest/qrdesktop.log' >> $WHERE
  echo >> $WHERE
  echo ':app-name,contains,"pnserver"	~	-/var/log/messages' >> $WHERE
  echo ':app-name,contains,"pntsc"	~	-/var/log/messages' >> $WHERE
  echo ':app-name,contains,"qrdesktop"	~	-/var/log/messages' >> $WHERE
  echo >> $WHERE

killall -HUP rsyslogd

else
echo 'Loggin already installed.'
fi

echo
echo '======================================================='
echo 'Package build and installation complete'
echo '======================================================='
echo 

# performing post-installation checks
. ./postinstall.sh
