
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


ApplicationPage::ApplicationPage(ConnectionDialog *parent)
    : ConnectionDialogPage(parent)
{
    setTitle(tr("Startup application"), tr("Specify the startup applications options."));
    label1 = new QLabel ("Grant the user access to the following:");
    label2 = new QLabel ("Specify the desired published desktop or application to start on logon.");
    appsLabel = new QLabel(tr("Published Desktop or Application Name:"));
    appsLabel->setEnabled(false);
	grantDesktopsCheckBox = new QCheckBox("Published desktops");
    grantDesktopsCheckBox->setChecked(false);
	grantAppsCheckBox = new QCheckBox("Published applications");
    grantAppsCheckBox->setChecked(false);
    promptRadioButton = new QRadioButton("Prompt the user for the desired published desktop or application");
    promptRadioButton->setChecked(true);
    specifyRadioButton = new QRadioButton("Specify the desired desktop or application to be started");
    appsComboBox = new QComboBox;
	appsComboBox->setEditable(true);
	appsComboBox->setEnabled(false);
    appNameLineEdit = new QLineEdit;
    appNameLineEdit->setVisible(false);
    appNameLineEdit->setEnabled(false);
	MyPushButtonCreator creator;
    appsRefreshButton = creator.createButton("Refresh");
    appsRefreshButton->setEnabled(false);

    registerField("alternate shell*", appNameLineEdit);
    registerField("grantDesktops", grantDesktopsCheckBox);
    registerField("grantApps", grantAppsCheckBox);
    registerField("saveStartupApp", specifyRadioButton);

    connect(specifyRadioButton, SIGNAL(toggled(bool)),
    		this, SLOT(setAppsEnabled(bool)));
    connect(appsComboBox, SIGNAL(currentIndexChanged(const QString&)),
    		appNameLineEdit, SLOT(setText(const QString&)));
    connect(appsRefreshButton, SIGNAL(clicked()), this, SLOT(refresh()));
    connect(appNameLineEdit, SIGNAL(textChanged(const QString&)),
    		this, SLOT(setAppName(const QString&)));
    connect(grantDesktopsCheckBox, SIGNAL(clicked(bool)),
            this, SLOT(checkFill(bool)));
    connect(grantAppsCheckBox, SIGNAL(clicked(bool)),
            this, SLOT(checkFill(bool)));

    parentDialog->setNextEnabled(false);    // both grants are unchecked

	QHBoxLayout *checkBoxesLayout = new QHBoxLayout;
	checkBoxesLayout->addWidget(grantDesktopsCheckBox);
	checkBoxesLayout->addWidget(grantAppsCheckBox);
    checkBoxesLayout->setContentsMargins(20, 10, 5, 30);

	QVBoxLayout *radioButtonsLayout = new QVBoxLayout;
	radioButtonsLayout->addWidget(promptRadioButton);
	radioButtonsLayout->addWidget(specifyRadioButton);
    radioButtonsLayout->setContentsMargins(20, 10, 5, 0);

    QGridLayout *appLayout = new QGridLayout;
    appLayout->addWidget(appsLabel, 0, 0);
    appLayout->addWidget(appsComboBox, 1, 0);
    appLayout->addWidget(appsRefreshButton, 1, 1);
    appLayout->setContentsMargins(30, 0, 5, 5);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addLayout(titleLayout);
    layout->addWidget(label1);
    layout->addLayout(checkBoxesLayout);
    layout->addWidget(label2);
    layout->addLayout(radioButtonsLayout);
    layout->addLayout(appLayout);
    layout->setAlignment(Qt::AlignTop);
    setLayout(layout);
}

//----------------------------------------------------------------------------
void ApplicationPage::refresh()
{
	int mask = (grantDesktopsCheckBox->isChecked() << 1) | grantAppsCheckBox->isChecked();
	if (!mask) return;
    QStringList apps(parentDialog->refreshAppsList(mask));
   	appsComboBox->clear();
    if (apps.size() != 0)
    {
   		for (int i = 0; i < apps.size(); i++)
   		{
   			appsComboBox->addItem(apps.at(i));
   		}
		appsComboBox->setCurrentIndex(0);
		if (appsComboBox->currentText() == "")
		{
			appsComboBox->removeItem(0);
		}
   	}

}

//----------------------------------------------------------------------------
void ApplicationPage::setAppName(const QString& name)
{
	if (appsComboBox->count() == 0)
	{
	    appsComboBox->addItem(name);
	    specifyRadioButton->setChecked(true);
	}
}

//----------------------------------------------------------------------------
void ApplicationPage::checkFill(bool granted)
{
    if(granted)
    {
        this->refresh();
        if (checkComplete() == 0)
        {
            parentDialog->setNextEnabled(true);
        }
    }
    else
    {
        if (!grantDesktopsCheckBox->isChecked() && !grantAppsCheckBox->isChecked())
        {
            parentDialog->setNextEnabled(false);
        }
    }
}

//----------------------------------------------------------------------------
bool ApplicationPage::checkComplete()
{
	if (appNameLineEdit->isEnabled() && appNameLineEdit->text() == "" ||
	    !grantDesktopsCheckBox->isChecked() && !grantAppsCheckBox->isChecked())
	{
		return 1;
	}
	return 0;
}

//----------------------------------------------------------------------------
void ApplicationPage::setAppsEnabled(bool enabled)
{
    if (enabled)
    {
        appsLabel->setEnabled(true);
    	appsComboBox->setEnabled(true);
        appNameLineEdit->setEnabled(true);
        appsRefreshButton->setEnabled(true);
        if (appNameLineEdit->text() == "")
        {
    	    parentDialog->setNextEnabled(false);
    	}
    }
    else
    {
        appsLabel->setEnabled(false);
    	appsComboBox->setEnabled(false);
    	appsComboBox->clear();
    	appsComboBox->clearEditText();
        appNameLineEdit->setText("");
        appNameLineEdit->setEnabled(false);
        appsRefreshButton->setEnabled(false);
        if (grantDesktopsCheckBox->isChecked() || grantAppsCheckBox->isChecked())
        {
            parentDialog->setNextEnabled(true);
        }
        promptRadioButton->setChecked(true);
    }
}
