
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef CONNWIZARD_H
#define CONNWIZARD_H

#include <QDialog>
#include <QMap>
#include <QFile>
#include <QProcess>
#include <QPushButton>
#include "optionsforms.h"

extern QString getPntscErrorString(int errcode);

//----------------------------------------------------------------------------
class QLabel;
class QListWidget;
class QStackedWidget;
class QFrame;
class QDialogButtonBox;

class ConnectionManagerMainWindow;

//----------------------------------------------------------------------------
class MyPushButtonCreator
{
public:
    QPushButton * createButton (const QString& text, int width = 0, int height = 0)
    {
        QPushButton *button = new QPushButton(text);
        button->setMinimumHeight(0);
        button->setMaximumHeight(height ? height : 25);
        button->setMinimumWidth(0);
        button->setMaximumWidth(width ? width : 100);
    	return button;
    }
};

//----------------------------------------------------------------------------
class ConnectionDialog : public QDialog
{
    Q_OBJECT

public:
    ConnectionDialog(QWidget *parent = 0);
	virtual void setNextEnabled(bool val){};
    virtual void accept();
	QStringList refreshAppsList(int mask);
	void setCredentials(const QString& username, const QString& password,
						const QString& domain);
    void addDomain(const QString& domain);
    QStringList domainList();
    void saveParameters(const QString& filename, const QStringList& parameters);
    void getParameters(const QString& filename, QStringList& parameters);

protected slots:
	void getPntscErrors();

protected:
	QString findField(const QString &name);

protected:
	QLabel * titleLabel;
	QDialogButtonBox *bottomButtonBox;
	QListWidget *leftList;
	QStackedWidget *pagesStackedWidget;
	QFrame *bottomLine;
	QProcess *pntsc;
	QMap < QString, QString > parameters;
// data fields
	QString username;
	QString password;
	QString domain;
	ConnectionManagerMainWindow *parentWindow;
};

//----------------------------------------------------------------------------
class ConnectionWizard : public ConnectionDialog
{
    Q_OBJECT

public:
    ConnectionWizard(QWidget *parent);
	void setNextEnabled(bool val);
    void accept();

private slots:
	void next();
	void back();
	void checkComplete(int pageIndex);

private:
	QPushButton *backButton;
	QPushButton *nextButton;
	QPushButton *cancelButton;
    ConnectionWizard(ConnectionWizard& other);
};

//----------------------------------------------------------------------------
class ConnectionEditor : public ConnectionDialog
{
    Q_OBJECT

public:
    ConnectionEditor(QString& pitFileName, QWidget *parent);
	void setNextEnabled(bool val);
    void accept();

private:
	QPushButton *okButton;
	ConnectionEditor();
	ConnectionEditor(const ConnectionEditor& other);
};


#endif
