
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


CredentialsPage::CredentialsPage(ConnectionDialog *parent)
    : ConnectionDialogPage(parent)
{
	QFont font;
	font.setBold(true);

	setTitle(tr("Credentials"), tr("Specify the credentials to be used to "
								 "authenticate to the farm and receive the "
								 "authorized list of published desktops and applications."));

    userNameLabel = new QLabel(tr("Username:"));
    userNameLineEdit = new QLineEdit;

    passwordLabel = new QLabel(tr("Password:"));
    passwordLineEdit = new QLineEdit;
    passwordLineEdit->setEchoMode(QLineEdit::Password);

    domainLabel = new QLabel(tr("Domain:"));
//    domainLineEdit = new QLineEdit;
    domainComboBox = new QComboBox;

    reuseRadioButton = new QRadioButton("Reuse these credentials whenever this connection is started");
    promptRadioButton = new QRadioButton("Always prompt for credentials whenever this connection is started");
    promptRadioButton->setChecked(true);
    userNameLabel->setEnabled(false);
    passwordLabel->setEnabled(false);
    domainLabel->setEnabled(false);
    userNameLineEdit->setEnabled(false);
    passwordLineEdit->setEnabled(false);
//    domainLineEdit->setEnabled(false);
    domainComboBox->setEditable(true);
    domainComboBox->setEnabled(false);
    domainComboBox->setDuplicatesEnabled(false);
    
    registerField("username", userNameLineEdit);
    registerField("password", passwordLineEdit);
//    registerField("domain*", domainLineEdit);
    registerField("domain", domainComboBox->lineEdit());
    registerField("saveCredentials", reuseRadioButton);

    QStringList domainList(parentDialog->domainList());
    foreach(QString domain, domainList)
        domainComboBox->addItem(domain);
    domainComboBox->setCurrentIndex(0);

    connect(reuseRadioButton, SIGNAL(toggled(bool)),
            this, SLOT(checkCredentials(bool)));
    connect(userNameLineEdit, SIGNAL(textChanged(const QString&)),
	    this, SLOT(setCredentials()));
    connect(passwordLineEdit, SIGNAL(textChanged(const QString&)),
	    this, SLOT(setCredentials()));
//	connect (domainLineEdit, SIGNAL(textChanged(const QString&)),
//			 this, SLOT(checkCompleteness()));
    connect (domainComboBox->lineEdit(), SIGNAL(editingFinished()),
             this, SLOT (setCredentials()));
    connect (domainComboBox->lineEdit(), SIGNAL(editingFinished()),
             this, SLOT (addDomain()));

    QGridLayout *gridLayout = new QGridLayout;
    gridLayout->setColumnMinimumWidth(0, 120);
    gridLayout->addWidget(userNameLabel, 0, 0);
    gridLayout->addWidget(userNameLineEdit, 0, 1);
    gridLayout->addWidget(passwordLabel, 1, 0);
    gridLayout->addWidget(passwordLineEdit, 1, 1);
    gridLayout->addWidget(domainLabel, 2, 0);
//    gridLayout->addWidget(domainLineEdit, 2, 1);
    gridLayout->addWidget(domainComboBox, 2, 1);
    gridLayout->setContentsMargins(10, 10, 20, 10);
    QVBoxLayout *radioLayout = new QVBoxLayout;
    radioLayout->addWidget(reuseRadioButton);
    radioLayout->addWidget(promptRadioButton);
    radioLayout->setContentsMargins(10, 10, 20, 10);
    radioLayout->setSpacing(5);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addLayout(titleLayout);
    layout->addLayout(gridLayout);
    layout->addLayout(radioLayout);
    layout->setAlignment(Qt::AlignTop);
    setLayout(layout);
}

//----------------------------------------------------------------------------
void CredentialsPage::setCredentials()
{
    if (!reuseRadioButton->isChecked())
    {
	reuseRadioButton->toggle();
    }
}

//----------------------------------------------------------------------------
void CredentialsPage::checkCredentials(bool credsAreNecessary)
{
    if (credsAreNecessary)
    {
        userNameLineEdit->setEnabled(true);
        passwordLineEdit->setEnabled(true);
//        domainLineEdit->setEnabled(true);
        domainComboBox->setEnabled(true);
        userNameLabel->setEnabled(true);
        passwordLabel->setEnabled(true);
        domainLabel->setEnabled(true);
    }
    else
    {
        userNameLineEdit->setText("");
        passwordLineEdit->setText("");
//        domainLineEdit->setText("");
        userNameLineEdit->setEnabled(false);
        passwordLineEdit->setEnabled(false);
//        domainLineEdit->setEnabled(false);
        domainComboBox->setEnabled(false);
        userNameLabel->setEnabled(false);
        passwordLabel->setEnabled(false);
        domainLabel->setEnabled(false);
        parentDialog->setNextEnabled(true);
        promptRadioButton->setChecked(true);
    }
}

//----------------------------------------------------------------------------
void CredentialsPage::addDomain()
{
//    if (domainComboBox->findText(domain) == -1)
//    {
//        domainComboBox->addItem(domain);
        parentDialog->addDomain(domainComboBox->currentText());
 //   }
}
