
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef DIALOGPAGES_H
#define DIALOGPAGES_H

#include <QDialog>
#include <QMap>
#include <QtGui>
#include "conndialog.h"


//----------------------------------------------------------------------------
class QCheckBox;
class QGroupBox;
class QRadioButton;
class QTextEdit;
class QLabel;
class QLineEdit;
class QComboBox;
class QListWidget;
class QFrame;
//class QVBoxLayout;
class QStandardItemModel;
class QTableWidget;
class QModelIndex;
class QSpinBox;

class ConnectionDialog;


//----------------------------------------------------------------------------
class ConnectionDialogPage : public QWidget
{
    Q_OBJECT

public:
	ConnectionDialogPage(ConnectionDialog *parent);
	QMap < QString, QWidget * >::const_iterator firstField();
	QMap < QString, QWidget * >::const_iterator endField();
	virtual void setField(const QString& name, const QString& value);
	virtual bool checkComplete();

protected slots:
	void setCompleted(const QString& text);

protected:
	void setTitle (const QString& titleText, const QString& subTitleText = "");
	void registerField(QString fieldName,  QWidget* fieldWidget);

	QFrame *titleFrame;
	QLabel *titleText;
	QLabel *subTitleText;
	QVBoxLayout *titleLayout;
	int registerMask;
	int completeMask;
	ConnectionDialog *parentDialog;

private:
	ConnectionDialogPage();
	QMap < QString, QWidget * > fields;
};

//----------------------------------------------------------------------------
class IntroPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    IntroPage(ConnectionDialog *parent);

private:
    QLabel *label;
};

//----------------------------------------------------------------------------
class SessionNamePage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    SessionNamePage(ConnectionDialog *parent);

private slots:
    void checkName();
    void sessionNameChanged(const QString &text);

private:
    QLabel *sessionNameLabel;
    QLineEdit *sessionNameLineEdit;
    QString currentSessionName;
};

//----------------------------------------------------------------------------
class CredentialsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    CredentialsPage(ConnectionDialog *parent);

private slots:
    void checkCredentials(bool);
    void setCredentials();
    void addDomain();

private:
    QLabel *userNameLabel;
    QLabel *passwordLabel;
    QLabel *domainLabel;
    QLineEdit *userNameLineEdit;
    QLineEdit *passwordLineEdit;
//    QLineEdit *domainLineEdit;
    QComboBox *domainComboBox;
    QRadioButton *reuseRadioButton;
    QRadioButton *promptRadioButton;
};

//----------------------------------------------------------------------------
class ConnectionPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    ConnectionPage(ConnectionDialog *parent);

private:
    void checkBrokerListState();
    void updateBrokerLineEdit();

private slots:
	void addBroker();
	void removeBroker();
	void editItem(const QModelIndex &index);
	void setBrokerList(const QString& brokers);
	void setSslEnabled(bool);
	void upBroker();
	void downBroker();

private:
    QGroupBox *brokerOptionsGroupBox;
    QLabel *brokersLabel;
    QListWidget *brokersList;
    QLineEdit *brokerLineEdit;
    QLabel *protocolLabel;
    QComboBox *protocolComboBox;
    QLabel *portLabel;
    QLineEdit *portLineEdit;
    QPushButton *addBrokerButton;
    QPushButton *removeBrokerButton;
    QPushButton *downBrokerButton;
    QPushButton *upBrokerButton;
    QGroupBox *connectionOptionsGroupBox;
    QCheckBox *natEnabledCheckBox;
    QCheckBox *sslEnabledCheckBox;
    QLabel *sslGatewayLabel;
    QLineEdit *sslGatewayLineEdit;
    QLabel *waitLabel;
    QLabel *retryLabel;
    QSpinBox *waitSpinBox;
    QSpinBox *retrySpinBox;
    QValidator *portValidator;
};

//----------------------------------------------------------------------------
class ApplicationPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    ApplicationPage(ConnectionDialog *parent);
	virtual bool checkComplete();

private slots:
	void refresh();
	void setAppName(const QString& name);
	void checkFill (bool granted);
	void setAppsEnabled(bool enabled);

private:
    QLabel *label1;
    QLabel *label2;
    QLabel *appsLabel;
    QCheckBox *grantDesktopsCheckBox;
    QCheckBox *grantAppsCheckBox;
    QRadioButton *promptRadioButton;
    QRadioButton *specifyRadioButton;
    QComboBox *appsComboBox;
    QLineEdit *appNameLineEdit;
    QPushButton *appsRefreshButton;
};

//----------------------------------------------------------------------------
class DisplayOptionsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    DisplayOptionsPage(ConnectionDialog *parent);

private slots:
    void setWindowSize(const QString& text);
    void geometryChanged(const QString& text);

private:
	QLabel *label1;
	QLabel *windowSizeLabel;
	QLabel *customHeightLabel;
	QLabel *customWidthLabel;
	QLabel *colorQualityLabel;
	QComboBox *windowSizeComboBox;
	QComboBox *colorQualityComboBox;
	QLineEdit *customHeightLineEdit;
	QLineEdit *customWidthLineEdit;
	QCheckBox *checkBox1;
	QCheckBox *checkBox2;
	QIntValidator *validator;
};

//----------------------------------------------------------------------------
class ClientOptionsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    ClientOptionsPage(ConnectionDialog *parent);

private slots:
    void enableSound(bool enable);
    void setSoundCheckBox(const QString &text);

private:
	QLabel *clientNameLabel;
	QLabel *keyboardLayoutLabel;
	QLabel *noteLabel;
	QLineEdit *clientNameLineEdit;
	QLineEdit *soundLineEdit;
	QComboBox *keyboardLayoutComboBox;
	QCheckBox *soundCheckBox;
	QCheckBox *enableMmrCheckBox;
	QCheckBox *enableFmmrCheckBox;
	QCheckBox *enableUsbrdrCheckBox;
	QCheckBox *enableGaCheckBox;
	QCheckBox *sendingMouseCheckBox;

	QGroupBox *downstreamAudioGroupBox;
	QGroupBox *upstreamAudioGroupBox;
	QCheckBox *daEnableCheckBox;
	QCheckBox *uaEnableCheckBox;
	QComboBox *daModeComboBox;
	QComboBox *daQualiltyComboBox;
	QLineEdit *daCustomQualityLineEdit;
	QLineEdit *daBufferingLineEdit;
	QComboBox *uaModeComboBox;
	QLineEdit *uaQualityLineEdit;
	QLineEdit *uaComplexityLineEdit;
	QCheckBox *uaEnhancedCheckBox;
};

//----------------------------------------------------------------------------
class DrivesMappingOptionsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    DrivesMappingOptionsPage(ConnectionDialog *parent);

private slots:
	void addItem();
	void removeItem();
	void removeAllItem();
	void editItem(const QModelIndex &index);
	void populateList(const QString& items);

private:
    void checkList();

private:
    QLabel *label;
    QListWidget *list;
    QLineEdit *lineEdit;
    QPushButton *addButton;
    QPushButton *removeButton;
    QPushButton *removeAllButton;
};

//----------------------------------------------------------------------------
class LPTPortsMappingOptionsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    LPTPortsMappingOptionsPage(ConnectionDialog *parent);

private slots:
	void addItem();
	void removeItem();
	void removeAllItem();
	void editItem(const QModelIndex &index);
	void populateList(const QString& items);

private:
    QLabel *label;
    QListWidget *list;
    QLineEdit *lineEdit;
    QPushButton *addButton;
    QPushButton *removeButton;
    QPushButton *removeAllButton;
};

//----------------------------------------------------------------------------
class COMPortsMappingOptionsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    COMPortsMappingOptionsPage(ConnectionDialog *parent);

private slots:
	void addItem();
	void removeItem();
	void removeAllItem();
	void editItem(const QModelIndex &index);
	void populateList(const QString& items);

private:
    QLabel *label;
    QListWidget *list;
    QLineEdit *lineEdit;
    QPushButton *addButton;
    QPushButton *removeButton;
    QPushButton *removeAllButton;
};

//----------------------------------------------------------------------------
class RDPPrintersOptionsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    RDPPrintersOptionsPage(ConnectionDialog *parent);

	virtual void setField(const QString& name, const QString& value);

private slots:
	void cellChanged(int row, int column);
	void editItem(int row, int column);

private:
    void parsePrintcap();

private:
    QLabel *label;
    QTableWidget *table;
    QLineEdit *lineEdit;
    QStringList printcapPrinters;
};

//----------------------------------------------------------------------------
class SmartCardsOptionsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    SmartCardsOptionsPage(ConnectionDialog *parent);
    bool toolTipEvent(QEvent *event);

private slots:
	void addItem();
	void removeItem();
	void removeAllItem();
	void editItem(const QModelIndex &index);
	void populateList(const QString& items);
	void setListEnabled(const int state);

private:
    void embraceWithQuotes(QString& text);

private:
	QCheckBox *checkBox;
    QLabel *label;
    QListWidget *list;
    QLineEdit *lineEdit;
    QPushButton *addButton;
    QPushButton *removeButton;
    QPushButton *removeAllButton;
};

//----------------------------------------------------------------------------
class ExperienceOptionsPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    ExperienceOptionsPage(ConnectionDialog *parent);

private:
	QCheckBox *backgroundCheckBox;
	QCheckBox *showWhileDraggingCheckBox;
	QCheckBox *animationCheckBox;
	QCheckBox *themesCheckBox;
	QCheckBox *bitmapCachingCheckBox;
};

//----------------------------------------------------------------------------
class StartupPage : public ConnectionDialogPage
{
    Q_OBJECT

public:
    StartupPage(ConnectionDialog *parent);

private:
    QCheckBox *autoStartCheckBox;
};


#endif
