
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


DisplayOptionsPage::DisplayOptionsPage(ConnectionDialog *parent)
	: ConnectionDialogPage(parent)
{
	setTitle(tr("Display Settings"), tr("Specify display settings for the connection."));

	label1 = new QLabel("Select the window size and color quality for this connection");
	windowSizeLabel = new QLabel("Window size");
	colorQualityLabel = new QLabel("Color quality");
	windowSizeComboBox = new QComboBox;
	windowSizeComboBox->addItem("fullscreen");
	windowSizeComboBox->addItem("640x480");
	windowSizeComboBox->addItem("800x600");
	windowSizeComboBox->addItem("1024x768");
	windowSizeComboBox->addItem("custom");
    validator = new QIntValidator (200, 65535, this);
    customHeightLabel = new QLabel(tr("Custom height:"));
    customHeightLineEdit = new QLineEdit;
    customHeightLineEdit->setEnabled(false);
    customHeightLineEdit->setValidator(validator);
    customWidthLabel = new QLabel(tr("     Custom width:"));
    customWidthLineEdit = new QLineEdit;
    customWidthLineEdit->setEnabled(false);
    customWidthLineEdit->setValidator(validator);
	colorQualityComboBox = new QComboBox;
	colorQualityComboBox->addItem("auto");
	colorQualityComboBox->addItem("8");
	colorQualityComboBox->addItem("15");
	colorQualityComboBox->addItem("16");
	colorQualityComboBox->addItem("24");
	checkBox1 = new QCheckBox("Use off-screen backup window");
	checkBox2 = new QCheckBox("Span multiple monitors when in full-screen mode");

    connect(windowSizeComboBox, SIGNAL(currentIndexChanged(const QString &)),
            this, SLOT(setWindowSize(const QString &)));
    connect( customHeightLineEdit, SIGNAL( textChanged( const QString & ) ),
             this, SLOT( geometryChanged( const QString & ) ) );
    connect( customWidthLineEdit, SIGNAL( textChanged( const QString & ) ),
             this, SLOT( geometryChanged( const QString & ) ) );

	registerField("windowSize", windowSizeComboBox);
	registerField("customWindowHeight", customHeightLineEdit);
	registerField("customWindowWidth", customWidthLineEdit);
	registerField("session bpp", colorQualityComboBox);
	registerField("backingStore", checkBox1);
	registerField("spanmonitors", checkBox2);

	QGridLayout *gridLayout = new QGridLayout;
	gridLayout->addWidget(colorQualityLabel, 0, 0);
	gridLayout->addWidget(colorQualityComboBox, 0, 1);
	gridLayout->addWidget(windowSizeLabel, 1, 0);
	gridLayout->addWidget(windowSizeComboBox, 1, 1);
	gridLayout->setContentsMargins(0, 5, 30, 0);

	QHBoxLayout *hLayout1 = new QHBoxLayout;
	hLayout1->addWidget(customHeightLabel);
	hLayout1->addWidget(customHeightLineEdit);
	hLayout1->addWidget(customWidthLabel);
	hLayout1->addWidget(customWidthLineEdit);
	hLayout1->setContentsMargins(80, 0, 30, 20);
	hLayout1->setSpacing(20);

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addLayout(titleLayout);
	layout->addWidget(label1);
	layout->addLayout(gridLayout);
	layout->addLayout(hLayout1);
	layout->addWidget(checkBox1);
	layout->addWidget(checkBox2);
    layout->setAlignment(Qt::AlignTop);
	setLayout(layout);
}

void DisplayOptionsPage::geometryChanged(const QString& text)
{
    parentDialog->setNextEnabled( this->checkComplete() == 0 );
}

void DisplayOptionsPage::setWindowSize(const QString& text)
{
    if (text == "custom")
    {
        customHeightLabel->setEnabled(true);
        customHeightLineEdit->setEnabled(true);
        customWidthLabel->setEnabled(true);
        customWidthLineEdit->setEnabled(true);
    }
    else
    {
        customHeightLabel->setEnabled(false);
        customHeightLineEdit->setEnabled(false);
        customWidthLabel->setEnabled(false);
        customWidthLineEdit->setEnabled(false);
    }
}
