
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include <qmessagebox.h>
#include <fcntl.h>
#include <cstdio>
#include <QApplication>
#include <QPlastiqueStyle>

#include "mainwindow.h"

#define LOCKFILENAME        "/tmp/.provision_connmanager_lock"

static int fdlock;

int get_lock(void)
{
    struct flock fl;

    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    fl.l_start = 0;
    fl.l_len = 1;

    if((fdlock = open( LOCKFILENAME, O_WRONLY|O_CREAT, 0666)) == -1)
        return 0;

    if(fcntl(fdlock, F_SETLK, &fl) == -1)
        return 0;

    return 1;
}

void remove_lock(void)
{
    close(fdlock);
    remove(LOCKFILENAME);
}


//----------------------------------------------------------------------------
int main(int argc, char *argv[])
{
    int rc, rc2;

    Q_INIT_RESOURCE(connmanager);

    QApplication::setDesktopSettingsAware(true);

    QApplication app(argc, argv);

    if (argc > 1)
    {
	    // start the connection from the given file
	    strcpy(argv[0], "pntsc");
	    argv[0][6] = 0;
	    if (execvp(argv[0], argv) < 0)
	    {
	        fprintf (stderr, "Cannot start the connection - launching the pntsc utility failed\n");
	        exit(1);
	    }
    }

    // Attempt to get the execution lock.
    // If this fails then another instance is running somewhere.
    rc = get_lock();

    // rc != 0 means that the lock was obtained.
    if ( rc != 0 )
    {
       ConnectionManagerMainWindow mainWindow;

       QObject::connect(&app, SIGNAL(aboutToQuit()), &mainWindow, SLOT(aboutToQuit()));
       mainWindow.setWindowTitle( APPLICATION_NAME );
       mainWindow.show();

       rc2 = app.exec();

        remove_lock();

       return rc2;
    }
    else
    {
        // The lock file is still present, so cannot run, instead, throw the
        // error and display the test in a message window
        QMessageBox::warning( 0, "Warning", "Another instance of this application is already running", QMessageBox::Ok, QMessageBox::NoButton );

        return -2;
    }

}
