
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QDialog>
#include <QProcess>
#include <QHash>
#include <QtXml/QXmlStreamReader>
#include <QtXml/QXmlStreamWriter>

#define CONFIG_FILE_NAME     "pnconfig.txt"
#define XML_CONFIG_FILE_NAME ".qrdesktop.cfg"
#define XML_CONFIG_PATH      ".connmanager"
#define APPLICATION_NAME     "Quest vWorkspace Connector"

//----------------------------------------------------------------------------
class QAction;
class QMenu;
class QLabel;
class QListWidget;
class QLineEdit;
class QComboBox;
class QPushButton;
class QDialogButtonBox;
class QTextEdit;
class QProcess;
class QCheckBox;
class QListWidgetItem;

//----------------------------------------------------------------------------
class ConnectionManagerMainWindow : public QMainWindow
{
	Q_OBJECT

public:
	ConnectionManagerMainWindow();
	void setCredentials(const QString& username, const QString& password,
						const QString& domain);
	void setApplicationName(const QString& appName);
	void addDomain(const QString& domain);
    QStringList domainList();

public slots:
    void aboutToQuit();

protected:
	void contextMenuEvent(QContextMenuEvent *event);
	void closeEvent(QCloseEvent *event);

private slots:
	void changeDir();
	void newConnection();
	void editConnection();
	void deleteConnection();
	void startConnection();
	void showHelp();
	void readChildErrors();
	void readChildOutput();
	void setAutoLaunchConnection(bool);
	void checkAutoLaunch(QListWidgetItem *, QListWidgetItem *);

private:
    void createActions();
    void createMenus();
    void createToolBars();
    void enableConnectionActions();
    void disableConnectionActions();
    void readAutoLaunchedApps(QXmlStreamReader *xml);
    void readDomains(QXmlStreamReader *xml);

private:
	QLabel *label;
	QListWidget *connList;
	QCheckBox *autoLaunchCheckBox;

	QMenu *fileMenu;
	QMenu *connMenu;
	QMenu *helpMenu;
	QToolBar *toolBar;
	QAction *changeDirAction;
	QAction *newAction;
	QAction *editAction;
	QAction *startAction;
	QAction *deleteAction;
	QAction *exitAction;
	QAction *helpAction;

	QString broker;
	QString username;
	QString password;
	QString domain;
	QString appName;
	QString currentStatusMessage;
	QString currentConnectionName;
	QString workingDirName;
	QString autoLaunchConnectionName;

	QStringList domains;
	QStringList autoLaunchedApps;

    QProcess *pntsc;
};


#endif
