
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include <QtGui>
#include "optionsforms.h"
#include "mainwindow.h"
#include "conndialog.h"
#include "../common/code.h"


//----------------------------------------------------------------------------
//		Additional dialogs
//----------------------------------------------------------------------------
CredentialsForm::CredentialsForm(QWidget *parent)
	: QDialog(),
	  parentWindow(parent)
{
	usernameLabel = new QLabel("Username:");
	passwordLabel = new QLabel("Password:");
	domainLabel = new QLabel("Domain:");
	usernameLineEdit = new QLineEdit;
	passwordLineEdit = new QLineEdit;
	passwordLineEdit->setEchoMode(QLineEdit::Password);
	domainComboBox = new QComboBox;
	domainComboBox->setEditable(true);
    domainComboBox->setDuplicatesEnabled(false);
//	domainLineEdit = new QLineEdit;
	bottomButtonBox = new QDialogButtonBox(Qt::Horizontal);
	MyPushButtonCreator creator;
	okButton = creator.createButton("Accept");
	okButton->setEnabled(false);
	cancelButton = creator.createButton("Cancel");
    bottomButtonBox->addButton(okButton, QDialogButtonBox::AcceptRole);
    bottomButtonBox->addButton(cancelButton, QDialogButtonBox::RejectRole);
    setTabOrder(okButton, cancelButton);

	ConnectionManagerMainWindow *cmWindow;
	ConnectionDialog *cdWindow;
    QStringList domainList;
	if((cmWindow = qobject_cast <ConnectionManagerMainWindow *> (parentWindow)) != NULL)
	{
    	domainList = cmWindow->domainList();
	}
	else if ((cdWindow = qobject_cast <ConnectionDialog *> (parentWindow)) != NULL)
	{
    	domainList = cdWindow->domainList();
	}

    QString domain;
    foreach(domain, domainList)
        domainComboBox->addItem(domain);

    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
	connect (usernameLineEdit, SIGNAL(textChanged(const QString&)),
			 this, SLOT(checkCompleteness()));
	connect (passwordLineEdit, SIGNAL(textChanged(const QString&)),
			 this, SLOT(checkCompleteness()));
	connect (domainComboBox->lineEdit(), SIGNAL(textChanged(const QString&)),
			 this, SLOT(checkCompleteness()));
    connect (domainComboBox->lineEdit(), SIGNAL(editingFinished()),
             this, SLOT (addDomain()));

    QGridLayout *gridLayout = new QGridLayout;
    gridLayout->addWidget(usernameLabel, 0, 0);
    gridLayout->addWidget(passwordLabel, 1, 0);
    gridLayout->addWidget(domainLabel, 2, 0);
    gridLayout->addWidget(usernameLineEdit, 0, 1);
    gridLayout->addWidget(passwordLineEdit, 1, 1);
//    gridLayout->addWidget(domainLineEdit, 2, 1);
    gridLayout->addWidget(domainComboBox, 2, 1);

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addLayout(gridLayout);
	layout->addWidget(bottomButtonBox);
	setLayout(layout);
	setWindowTitle("Set credentials");
}

//----------------------------------------------------------------------------
void CredentialsForm::checkCompleteness()
{
	if (usernameLineEdit->text() == "" ||
		passwordLineEdit->text() == "" ||
//		domainLineEdit->text() == "")
		domainComboBox->currentText() == "")
	{
		okButton->setEnabled(false);
	}
	else
	{
		okButton->setEnabled(true);
	}
}

//----------------------------------------------------------------------------
void CredentialsForm::addDomain()
{
	ConnectionManagerMainWindow *cmWindow;
	ConnectionDialog *cdWindow;

//    if (domainComboBox->findText(domain) == -1)
//    {
//        domainComboBox->addItem(domain);
    	if((cmWindow = qobject_cast <ConnectionManagerMainWindow *> (parentWindow)) != NULL)
	    {
    	    cmWindow->addDomain(domainComboBox->currentText());
    	}
	    else if ((cdWindow = qobject_cast <ConnectionDialog *> (parentWindow)) != NULL)
    	{
        	cdWindow->addDomain(domainComboBox->currentText());
	    }
//    }
}

//----------------------------------------------------------------------------
void CredentialsForm::accept()
{
	ConnectionManagerMainWindow *cmWindow;
	ConnectionDialog *cdWindow;

	if((cmWindow = qobject_cast <ConnectionManagerMainWindow *> (parentWindow)) != NULL)
	{
    	cmWindow->setCredentials(usernameLineEdit->text(),
    						     passwordLineEdit->text(),
//    						     domainLineEdit->text());
    						     domainComboBox->currentText());
	}
	else if ((cdWindow = qobject_cast <ConnectionDialog *> (parentWindow)) != NULL)
	{
    	cdWindow->setCredentials(usernameLineEdit->text(),
    						     passwordLineEdit->text(),
//    						     domainLineEdit->text());
    						     domainComboBox->currentText());
	}
	QDialog::accept();
}

//----------------------------------------------------------------------------
QString CredentialsForm::getUsername()
{
    return usernameLineEdit->text();
}

//----------------------------------------------------------------------------
QString CredentialsForm::getPassword()
{
    return passwordLineEdit->text();
}

//----------------------------------------------------------------------------
QString CredentialsForm::getDomain()
{
    return domainComboBox->currentText();
}

//----------------------------------------------------------------------------
ApplicationForm::ApplicationForm(QWidget *parent,
								 const QString& b, const QString& u,
								 const QString& p, const QString& d, const QString& l)
	: parentWindow(parent), pntsc(0), broker(b), username(u), password(p),
	  domain(d), list(l), rejectRequest(0), errorStr("")

{
	label = new QLabel("Published desktop or application name:");
	appsComboBox = new QComboBox;
	appsComboBox->setEditable(true);
	MyPushButtonCreator creator;
	refreshButton = creator.createButton("Refresh");
	bottomButtonBox = new QDialogButtonBox;
	okButton = creator.createButton("Accept");
	okButton->setEnabled(false);
	cancelButton = creator.createButton("Cancel");
    bottomButtonBox->addButton(okButton, QDialogButtonBox::AcceptRole);
    bottomButtonBox->addButton(cancelButton, QDialogButtonBox::RejectRole);
    setTabOrder(okButton, cancelButton);
    statusBar = new QStatusBar(this);
   	statusBar->showMessage("Ready");

    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
    connect(refreshButton, SIGNAL(clicked()), this, SLOT(refreshAppList()));
	connect (appsComboBox, SIGNAL(editTextChanged(const QString&)),
			 this, SLOT(checkCompleteness(const QString&)));
	connect (appsComboBox, SIGNAL(currentIndexChanged(const QString&)),
			 this, SLOT(checkCompleteness(const QString&)));

    QGridLayout *gridLayout = new QGridLayout;
    gridLayout->addWidget(label, 0, 0);
    gridLayout->addWidget(appsComboBox, 1, 0);
    gridLayout->addWidget(refreshButton, 1, 1);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addLayout(gridLayout);
    layout->addWidget(bottomButtonBox);
    layout->addWidget(statusBar);
    setLayout(layout);
    setWindowTitle("Set startup application");
}

//----------------------------------------------------------------------------
void ApplicationForm::refreshAppList()
{
    do
    {
        errorStr = "";
        if (pntsc != NULL)
            return;
        appsComboBox->clear();
        apps.clear();
        pntsc = new QProcess;
        connect (pntsc, SIGNAL(readyReadStandardError()),
                 this, SLOT(getErrors()));
        connect (pntsc, SIGNAL(readyReadStandardOutput()),
                 this, SLOT(getOutput()));
        pntsc->start("pntsc", QStringList() << broker << "-u" << ( username ) << "-p" << ( password )
                            << "-d" << ( domain ) << list);
        statusBar->showMessage("Getting an application list...");
        if (!pntsc->waitForStarted(1000))
        {
            QMessageBox::critical(this, "Getting an applications list", "Unable to start PNTSC");
        }
        else
        {
            while (pntsc->state() == QProcess::Running && !rejectRequest)
            {
                QApplication::processEvents();
            }
        }

        if (rejectRequest)
        {
            pntsc->terminate();
            pntsc->waitForFinished();
            delete pntsc;
            pntsc = 0;
            QDialog::reject();
            break;
        }
        else
        {
            delete pntsc;
            pntsc = 0;
            statusBar->showMessage("Ready");
            if (apps.size() != 0)
            {
                appsComboBox->clear();
                for (int i = 0; i < apps.size(); i++)
                {
                    appsComboBox->addItem(apps.at(i));
                }
            }
        }

        if (errorStr.contains(tr("Logon failure")))
        {
            CredentialsForm form(parentWindow);
            if (form.exec() == QDialog::Rejected)
            {
                return;
            }
            username = form.getUsername();
            password = form.getPassword();
            domain = form.getDomain();
        }

    } while(errorStr.contains(tr("Logon failure")));
    errorStr = "";
}

//----------------------------------------------------------------------------
void ApplicationForm::getErrors()
{
	QByteArray data = pntsc->readAllStandardError();

	// parse received error string
	QStringList list = QString(data).split("],");
	QString code = list.at(0);
	QString module = list.at(1);
	QString inf = errorStr = list.at(2);
    code.replace("[ERR","");
    module.replace("[","");

    bool ok;
    int errcode = code.toInt(&ok, 16);
    QString errcodestr = getPntscErrorString(errcode);

    QMessageBox::critical(this, "Getting an applications list",
        /*"[Error " + code + "]: " + errcodestr +
        "\n[Module]: " + module +
        "\n[Description]: " +*/ inf);
}

//----------------------------------------------------------------------------
void ApplicationForm::getOutput()
{
	QByteArray data = pntsc->readAllStandardOutput();
	QStringList list = QString(data).split("\n", QString::SkipEmptyParts);
    foreach (QString str, list)
    {
        if (str.startsWith("Name="))
        {
                // the application entry is given in the following format:
                // Name=%s[,FolderName=%s][,LargeIcon=%s][,SmallIcon=%s]
                str = str.remove("Name=");
                int pos = str.indexOf(",");
                if (pos != -1)
                {
                    // remove the tail with the unnecessary data
                    str = str.remove(pos, str.length() - pos);
                }
                apps.append(str);
        }
    }
}

//----------------------------------------------------------------------------
void ApplicationForm::checkCompleteness(const QString& text)
{
	if (text == "")
	{
		okButton->setEnabled (false);
	}
	else
	{
		okButton->setEnabled (true);
	}
}

//----------------------------------------------------------------------------
void ApplicationForm::accept()
{
	ConnectionManagerMainWindow *cmWindow;
	if((cmWindow = qobject_cast <ConnectionManagerMainWindow *> (parentWindow)) != NULL)
	{
    	cmWindow->setApplicationName(appsComboBox->currentText());
	}
	QDialog::accept();
}

//----------------------------------------------------------------------------
void ApplicationForm::reject()
{
	if (pntsc)
	{
		rejectRequest = 1;
	}
	else
	{
		QDialog::reject();
	}
}
