
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef OPTIONSFORMS_H
#define OPTIONSFORMS_H

#include <QDialog>

class QLabel;
class QLineEdit;
class QPushButton;
class QDialogButtonBox;
class QComboBox;
class QProcess;
class QStatusBar;


//----------------------------------------------------------------------------
class CredentialsForm : public QDialog
{
    Q_OBJECT

public:
	CredentialsForm(QWidget *parent);
	void accept();
	QString getUsername();
	QString getPassword();
	QString getDomain();

private slots:
	void checkCompleteness();
	void addDomain();

private:
	CredentialsForm();
	CredentialsForm(const CredentialsForm& other);

private:
	QLabel *usernameLabel;
	QLabel *passwordLabel;
	QLabel *domainLabel;
	QLineEdit *usernameLineEdit;
	QLineEdit *passwordLineEdit;
	QComboBox *domainComboBox;
//	QLineEdit *domainLineEdit;
	QDialogButtonBox *bottomButtonBox;
	QPushButton *okButton;
	QPushButton *cancelButton;
	QWidget *parentWindow;
};

//----------------------------------------------------------------------------
class ApplicationForm : public QDialog
{
    Q_OBJECT

public:
	ApplicationForm(QWidget *parent,
					const QString& broker, const QString& username,
					const QString& password, const QString& domain, const QString& list);
	void accept();
	void reject();

private slots:
	void checkCompleteness(const QString&);
	void refreshAppList();
	void getErrors();
	void getOutput();

private:
	ApplicationForm();
	ApplicationForm(const ApplicationForm& other);

private:
	QLabel *label;
	QComboBox *appsComboBox;
	QPushButton *refreshButton;
	QDialogButtonBox *bottomButtonBox;
	QPushButton *okButton;
	QPushButton *cancelButton;
	QStatusBar *statusBar;
	QWidget *parentWindow;
	QProcess *pntsc;
    QStringList apps;
	QString broker;
	QString username;
	QString password;
	QString domain;
	QString list;
	bool rejectRequest;
	QString autoStartConnectionName;
	QString errorStr;
};

#endif
