
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


RDPPrintersOptionsPage::RDPPrintersOptionsPage(ConnectionDialog *parent)
    : ConnectionDialogPage(parent)
{
    setTitle(tr("RDP Printers"),
    		 tr("Specify local printers to be redirected to the remote host."));

    label = new QLabel(tr("Printers:"));
    table = new QTableWidget (0,2);
    table->setEditTriggers(QAbstractItemView::DoubleClicked
                           | QAbstractItemView::SelectedClicked);
    table->setSelectionBehavior(QAbstractItemView::SelectRows);
    table->setShowGrid(0);
    QStringList headerLabels;
    headerLabels << "Printer" << "Driver";
    table->setHorizontalHeaderLabels(headerLabels);
    lineEdit = new QLineEdit;
    lineEdit->setVisible(false);

    // get printcap list of printers 
    parsePrintcap();

    // fill the table with the printers
    table->setRowCount(printcapPrinters.size());
	for (int i = 0; i < printcapPrinters.size(); i++)
	{
        QString full(printcapPrinters.at(i)), name, driver;
        int pos = full.indexOf("=");
	    name = full.left(pos);
        if (pos != -1)
        { 
    	    driver = full.remove(0, pos+1);
    	}
    	driver.remove("\"");
	    QTableWidgetItem *newItemName = new QTableWidgetItem(name);
	    QTableWidgetItem *newItemDriver = new QTableWidgetItem(driver);
	    newItemName->setFlags(Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
	    newItemDriver->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsEnabled);
	    table->setItem(i, 0, newItemName);
	    table->setItem(i, 1, newItemDriver);
        table->item(i, 0)->setCheckState(Qt::Unchecked);
    }
    table->resizeColumnsToContents();

    connect(table, SIGNAL(cellChanged(int, int)),
            this, SLOT(cellChanged(int, int)));
    connect(table, SIGNAL(cellDoubleClicked(int, int)),
            this, SLOT(editItem(int, int)));

	registerField("printers", lineEdit);
    
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addLayout(titleLayout);
    layout->addWidget(label);
    layout->addWidget(table);
//    layout->addWidget(lineEdit);
    layout->setAlignment(Qt::AlignTop);
    setLayout(layout);
}

//----------------------------------------------------------------------------
void RDPPrintersOptionsPage::setField(const QString& name, const QString& value)
{
	QStringList userPrinters = value.split(",", QString::SkipEmptyParts);
	
	// firstly replace all printcap drivernames with names from the PIT-file
	for (int i = 0; i < userPrinters.size(); i++)
	{
	    QString userPrinter = userPrinters.at(i), name, driver;
	    int pos = userPrinter.indexOf("=");
	    if (pos == -1)
	        continue;    // driver name is absent for this printer
	    name = userPrinter.left(pos);
        QStringList result = printcapPrinters.filter(name);
        if (result.isEmpty())
            continue;    // printer name is absent in the printcap file, skip

        // correct driver name using the name from the PIT-file
	    driver = userPrinter.remove(0, name.length()+1).remove("\"");
	    for (int j = 0; j < table->rowCount(); j++)
	    {
	        if (table->item(i, 0)->text() == name)
	        {
	            table->item(i, 1)->setText(driver);
	            break;
	        }
	    }
    }
    
    // now check printers which names are set in the PIT-file 
    for (int i = 0; i < table->rowCount(); i++)
    {
        foreach (QString str, userPrinters)
        {
            if (str.contains(table->item(i,0)->text()))
            {
                table->item(i, 0)->setCheckState(Qt::Checked);
        	    cellChanged(i, 0);
            }
        }
    }
}

//----------------------------------------------------------------------------
void RDPPrintersOptionsPage::cellChanged(int row, int column)
{
    QString name = table->item(row, 0)->text();
    QString driver = table->item(row, 1)->text();
    if (driver.contains("\""))
    {
        driver.remove("\"");
        table->item(row, 1)->setText(driver);
        return;
    }
    if (table->item(row,0)->checkState() == Qt::Unchecked)
    {
        // remove this printer from the list
        lineEdit->setText(lineEdit->text().remove(name));
        if (driver != "")
        {
            lineEdit->setText(lineEdit->text().remove("=\"" + driver + "\""));
        }
        if (lineEdit->text().startsWith(","))
            lineEdit->setText(lineEdit->text().remove(0,1));
        else if (lineEdit->text().endsWith(","))
            lineEdit->setText(lineEdit->text().remove(lineEdit->text().length()-1,1));
        else
            lineEdit->setText(lineEdit->text().replace(",,",","));
    }
    else
    {
        // add the printer to the list or correct its driver name
        if (lineEdit->text().indexOf(name) != -1)
        {
            // just correct the driver name
            int pos = lineEdit->text().indexOf(name) + name.length();

            // printf( "lineEdit = '%s' : name = '%s'\n",lineEdit->text().toAscii().data(), name.toAscii().data());

            if (lineEdit->text().indexOf("=") == -1 )
            {
                lineEdit->setText(lineEdit->text().insert(pos, "=\"" + driver + "\""));
            }
            if (lineEdit->text().at(pos) == '=')
            {
                // remove the old driver name
                lineEdit->setText(
                    lineEdit->text().remove(
                        pos, lineEdit->text().indexOf("\"", pos + 2) - pos + 1));
            }
            if (driver != "")
            {
                lineEdit->setText(lineEdit->text().insert(pos, "=\"" + driver + "\""));
            }
        }
        else
        {
            // add both printer and driver name
            if (lineEdit->text() != "")
            {
                lineEdit->setText(lineEdit->text().append(","));
            }
            lineEdit->setText(lineEdit->text().append(name));
            if (driver != "")
            {
                driver.remove("\"");
                table->item(row, 1)->setText(driver);
                lineEdit->setText(lineEdit->text().append("=\"" + driver + "\""));
            }
        }
    }
    table->resizeColumnsToContents();
}

//----------------------------------------------------------------------------
void RDPPrintersOptionsPage::editItem(int row, int column)
{
    if (column != 1)
    {
        table->editItem(table->item(row, 1));
    }
}

//----------------------------------------------------------------------------
void RDPPrintersOptionsPage::parsePrintcap()
{
	QFile file(QString("/etc/printcap"));    // default path in most Linux distrs

	if (!file.exists())
	{
	    file.setFileName("/var/run/cups/printcap");    // for Ubuntu
	    if (!file.exists())
	    {
            int ret = QMessageBox::warning(this, parentDialog->windowTitle(), 
                        tr("Can not found printcap file that is necessary to allow printers sharing\n"
                           "Would you like to try to find it? Otherwise printers sharing will be disabled."),
                           QMessageBox::Yes | QMessageBox::Ignore,
                           QMessageBox::Yes);
            if (ret == QMessageBox::Ignore)
            {
                return;
            }
            QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                     "/home",
                                                     tr("printcap file (printcap)"));
            if (fileName != "")
            {
                file.setFileName(fileName);
            }
            else
            {
                return;
            }
        }
	}

    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QMessageBox::critical(this, parentDialog->windowTitle(),"Error opening the printcap file"); 
        return;
    }

    QTextStream stream(&file);
    QString line;
    while (!stream.atEnd())
    {
        line = stream.readLine();
        line = line.trimmed();
        if (line.startsWith("#") || line.startsWith(":") || line == "") continue;
        
        // compound all lines ending with '\' in a single line
        while (line.endsWith("\\"))
        {
            if (stream.atEnd())
            {
                file.close();
                return;
            }
            line.remove(line.length()-1,1);
            QString line1 = stream.readLine();
            line1 = line1.trimmed();
            if (line1.startsWith("#")) continue;
            if (line1 == "")
            {
                file.close();
                return;
            }
            line = line + " " + line1;
        }
        
        // now we have one printcap entry in the form of a single string
        // so parse it - search printer and driver names
        int pos = line.indexOf("|");
        if (pos == -1)
        {
            pos = line.indexOf(":"); // can also equal -1
        }
        QString name = line.left(pos);
        QString driver;
        pos = line.indexOf(":wd=");
        if (pos != -1)
        {
            line.remove(0, pos + 4);
            driver = line.left(line.indexOf(":")); 
        }
        
        // add the printer name and driver to the list
        if (driver != "")
        {
            name = name + "=\"" + driver + "\"";
        }
        printcapPrinters.append(name); 
	}

	file.close();
}
