#!/bin/sh

BINARY_COMPONENTS="qrdesktop pntsc"
GST_COMPONENTS="pntssrc ximagesink"
test_component=
SUCCESS=0
GOOD_RETURN=1

test_gst_component()
{
  gst-inspect $@ 1>/dev/null 2>/dev/null
  return $?
}

test_binary_component()
{
  $@ 1>/dev/null 2>/dev/null
  if [ $? -eq $GOOD_RETURN ]
  then
    return 0
  else
    return 1
  fi
}

test_components()
{
  absent_components=""
  for component_name in $@
  do
    echo -n " checking for $component_name... "
    $test_component $component_name
    if [ $? -ne $SUCCESS ]
    then
      echo "  the component was not installed or doesn't work properly"
    else
      echo -e "\tOK"
      continue
    fi
    absent_components="$absent_components ${component_name}"
  done
  if [ "$absent_components" != "" ]
  then
    echo
    echo "  Absent components: $absent_components"
    echo
  fi
}

echo '======================================'
echo 'Performing post-installation checks...'
echo '======================================'
sleep 1

echo ''
echo 'Checking for binaries:'
test_component=test_binary_component
test_components $BINARY_COMPONENTS

if [ "$absent_components" != "" ]
then
  echo 
  echo "Some of binary components that are required were not properly installed."
  echo
else
  echo ' OK'
  echo "All binary components are available."
  echo
fi
sleep 1

echo ''
echo 'Checking for gstreamer packages:'
test_component=test_gst_component
test_components $GST_COMPONENTS

if [ "$absent_components" != "" ]
then
  echo 
  echo "Some of gstreamer components were not properly installed - MMR will unlikely work"
  echo
else
  echo ' OK'
  echo "All gstreamer components are available."
  echo
fi
sleep 1

echo
echo '============================='
echo 'Post-installation checks done'
echo '============================='
echo

