
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef _CHANNEL_TRANSPORT_H_
#define _CHANNEL_TRANSPORT_H_


#define TRANSPORT_EVENT_WAIT_TIMEOUT 0
#define TRANSPORT_EVENT_CONNECTED 1
#define TRANSPORT_EVENT_DISCONNECTED 2
#define TRANSPORT_EVENT_ACCEPTED 3
#define TRANSPORT_EVENT_MESSAGE_RECIEVED 4
#define TRANSPORT_EVENT_MESSAGE_SENT 5
#define TRANSPORT_EVENT_SYSTEM_ERROR 6
#define TRANSPORT_EVENT_EXIT 7

#define TRANSPORT_EVENT_COUNT 8

typedef int TRANSPORT_EVENT;

#ifdef DWORD
#undef DWORD
#endif

#ifndef VPG_OK
#define VPG_OK 0
#endif
#ifndef VPG_FAIL
#define VPG_FAIL -1
#endif

#define DWORD uint32

#define MAKE_SESSION_ID(x) ((x) ^ 0x80000000)
#define ISREMOTE_ID(x) ( ( (x) & 0x80000000) != 0 )

typedef uint32 session_id;
typedef int  vpg_result;


typedef vpg_result (* transport_event_handler ) ( TRANSPORT_EVENT _event );


typedef struct _channel_session
{
    session_id id;
    
    STREAM recieved;
    STREAM sending;
    
    void ( *channel_session_process ) ( struct _channel_session * _session, TRANSPORT_EVENT _ev );
    
}channel_session;

vpg_result channel_session_create_message ( channel_session * _session, STREAM * _msg , DWORD _size );
vpg_result channel_session_free_message   ( channel_session * _session, STREAM _msg );

vpg_result channel_session_recv_message   ( channel_session * _session, STREAM* _msg );
vpg_result channel_session_send_message   ( channel_session * _session, STREAM _msg );

vpg_result channel_transport_init ( char * _vchannelname , transport_event_handler _event_handler);

vpg_result channel_transport_create_session  ( channel_session ** _session );
vpg_result channel_transport_destroy_session ( channel_session * _session );


#endif
