
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "pnsslcli.h"
#include <cstdlib>
#include <syslog.h>
#include <sys/wait.h>
#include <signal.h>
#include <cassert>

typedef boost::shared_ptr < session > session_ptr;

extern char g_localaddr[64];
extern char g_ssladdr[64];
extern char g_remoteaddr[64];
extern char g_username[64];

// ------------------------------------------------------------------------------
//  main() arguments:
//  1) local loopback address <host[:port]>, default port is 3389
//  2) SSL VPN server address <host[:port]>, default port is 443
//  3) RDP hostname and 4) username, they're passed to VPN server
//      5) PNTSC pid, to say to that initialization is finished
// ------------------------------------------------------------------------------
extern "C"
void *
pnsslcli_thread (void * param)
{

    int err_code = 0;
    session_ptr work_session;
    
    syslog (LOG_NOTICE, "pnsslcli_thread: start session (g_ssladdr = %s)", g_ssladdr);

    try
    {
        asio::io_service io_service;
        asio::ssl::context ctx (io_service, asio::ssl::context::sslv23);
        work_session.reset (new session (io_service, ctx));
        ctx.set_verify_mode (asio::ssl::context::verify_peer);

        // connect to SSL VPN server
        char *p = strstr (g_ssladdr, ":");
        if (p)
        {
            *p++ = '\0';
        }
        tcp::resolver resolver (io_service);
        syslog (LOG_NOTICE, "g_ssladdr = %s", g_ssladdr );
        tcp::resolver::query query (tcp::v4 (), g_ssladdr, p ? p : "443");
        syslog (LOG_NOTICE, "Created query for %s:%s", g_ssladdr, p ? p : "443");
        tcp::resolver::iterator iterator = resolver.resolve (query);
        syslog (LOG_NOTICE, "Resolved query for %s:%s", g_ssladdr, p ? p : "443");
        work_session->sslsocket().lowest_layer().connect(*iterator);
        syslog (LOG_NOTICE, "connect to %s:%s is done", g_ssladdr, p ? p : "443");
        work_session->sslsocket().handshake(asio::ssl::stream_base::client);
        syslog (LOG_NOTICE, "handshake passed");
        syslog (LOG_NOTICE, "g_localaddr = %s", g_localaddr );

        // listen to local loopback
        short int port;
        p = strchr (g_localaddr, ':');
        if (p)
        {
            *p++ = '\0';
            port = std::atoi (p);
        }
        else
        {
            port = 3389;
        }
        
        syslog (LOG_NOTICE, "start listening %s:%d", g_localaddr, port);

        tcp::acceptor acceptor_ (io_service, 
                    tcp::endpoint (asio::ip::address::from_string (g_localaddr), port));
//        kill (getppid(), SIGUSR1);
        acceptor_.accept (work_session->socket ());
        syslog (LOG_NOTICE, "accept connection from %s:%d", g_localaddr, port);
        
        work_session->start (g_remoteaddr, g_username);
        
        io_service.run ();
        
    }
    catch (const asio::error_code & e)
    {
        if (e != asio::error::eof)
        {
            syslog (LOG_NOTICE, "error: %s", e.message().c_str());
//            work_session->socket().close();
//            work_session->sslsocket().lowest_layer().close();
        }
        else
        {
            syslog (LOG_NOTICE, "session was closed");
        }
        work_session.reset();
    }
    catch (std::exception & e)
    {
        syslog (LOG_NOTICE, "an exception was thrown: %s\n", e.what ());
//        work_session->socket().close();
//        work_session->sslsocket().lowest_layer().close();
        err_code = 1;
    }
    catch (...)
    {
        syslog (LOG_NOTICE, "an unknown exception was thrown\n");
    }

    syslog (LOG_NOTICE, "session was closed");
    closelog ();
    exit (1);
}
