
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef _QAUDIO_CODEC_H_
#define _QAUDIO_CODEC_H_

#define _FILE_OFFSET_BITS 64
#include <mpg123.h>

#include <speex/speex.h>
#include <speex/speex_stereo.h>


/*------------------------------speex encoder -----------------------------------------*/

typedef struct _speex_encoder
{
    WAVEFORMATEX format;
    void * encoder;

    short * data_buffer;
    uint32  data_buffer_size;

    SpeexBits bits;
} speex_encoder;


speex_encoder * codec_create ( LPWAVEFORMATEX format, int complexity, int quality );
void codec_destroy ( speex_encoder * codec );

uint32 codec_get_encoder_frame_bytes ( speex_encoder * codec );

uint32 codec_compress_buffer ( speex_encoder * codec , char * buffer , uint32 size , uint8 * out);

/*-------------------------------mp3 decoder------------------------------------------*/

struct _mp3decoder;

typedef int ( *pdecode ) (struct _mp3decoder * _decoder, unsigned char * _mp3, unsigned int _size , unsigned short * _out );

typedef struct _mp3decoder
{
    mpg123_handle * decoder;
    
    unsigned char data_buffer[ 16 ];
    unsigned int  start;
    unsigned int  end;


    pdecode decode;
    
} mp3decoder;


int mp3codec_initialize();
int mp3codec_uninitialize();

mp3decoder * mp3codec_create_decoder( int _rate , int _channels );
void mp3codec_destroy_decoder ( mp3decoder * _decoder );
int mp3codec_decode ( mp3decoder * _decoder, unsigned char * _mp3 , unsigned int _size, unsigned short * _out );



#endif
