
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "rdesktop.h"
#include "rdpsnd.h"
#include "qmic.h"

#include <speex/speex.h>
#include "qmic_codec.h"

static VCHANNEL * qmic_cmd_channel = NULL;
static VCHANNEL * qmic_data_channel = NULL;

extern struct audio_driver * current_driver;
extern RD_BOOL rec_device_open;

uint8  qmic_compression_support = True;
uint8  qmic_compression = False;

struct audio_driver * qmic_get_current_snd_driver()
{
    if ( current_driver == NULL ) return NULL;
    
    if ( ( current_driver->wave_in_open  == NULL )
       ||( current_driver->wave_in_close == NULL )
       ||( current_driver->wave_in_format_supported == NULL ) )
	{
    	    DEBUG_QMIC(("QMIC: Sound driver doesn't support recording!\n"));
	    return NULL;
	}
	
    return current_driver;
}

#define QMIC_MT_INTERNAL 0
#define QMIC_MT_WID      1

#define QMIC_CMD_HANDSHAKE 		0xC0DE
#define QMIC_CMD_GOODBYE   		0xED0C
#define QMIC_CMD_ENABLE_COMPRESSION	0xC011

#define QMIC_RPS_HANDSHAKE		0xDAAD
#define QMIC_RPS_GOODBYE		0xADDA

#define QMIC_OK 	0
#define QMIC_FAIL  -1

/* ++++++ redirection ++++++ */

typedef struct wavehdr_tag
{ 
    LPSTR      lpData; 
    DWORD      dwBufferLength; 
    DWORD      dwBytesRecorded; 
    DWORD_PTR  dwUser; 
    DWORD      dwFlags; 
    DWORD      dwLoops; 
    struct wavehdr_tag * lpNext; 
    DWORD_PTR reserved;
} WAVEHDR, *LPWAVEHDR; 

typedef struct
{
    WORD      wMid; 
    WORD      wPid; 
    MMVERSION vDriverVersion; 
    char      szPname[MAXPNAMELEN];
    DWORD     dwFormats; 
    WORD      wChannels; 
    WORD      wReserved1; 
} WAVEINCAPSA, *LPWAVEINCAPSA;

typedef struct
{
    WORD      wMid; 
    WORD      wPid; 
    MMVERSION vDriverVersion; 
    short     szPname[MAXPNAMELEN];
    DWORD     dwFormats; 
    WORD      wChannels; 
    WORD      wReserved1; 
} WAVEINCAPSW, *LPWAVEINCAPSW;

typedef void * LPWAVEINCAPS;

typedef struct mmtime_tag
{
    UINT            wType;      /* indicates the contents of the union */
    union
    {
        DWORD       ms;         /* milliseconds */
        DWORD       sample;     /* samples */
        DWORD       cb;         /* byte count */
        DWORD       ticks;      /* ticks in MIDI stream */
                                    
        /* SMPTE */
	struct
        {
    	    BYTE    hour;       /* hours */
            BYTE    min;        /* minutes */
            BYTE    sec;        /* seconds */
            BYTE    frame;      /* frames  */
            BYTE    fps;        /* frames per second */
            BYTE    dummy;      /* pad */
            BYTE    pad[2];
        } smpte;
        /* MIDI */
        struct
        {
            DWORD songptrpos;   /* song pointer position */
        } midi;
    } u;
} MMTIME, *PMMTIME, *NPMMTIME, *LPMMTIME;

static UINT     waveInGetNumDevs( void );
static MMRESULT waveInGetDevCaps( UINT_PTR uDeviceID, LPWAVEINCAPS pwic, UINT cbwic );
static MMRESULT waveInClose( HWAVEIN hwi );
static MMRESULT waveInOpen( LPHWAVEIN phwi, UINT_PTR uDeviceID, LPWAVEFORMATEX pwfx,DWORD_PTR dwCallback, DWORD_PTR dwCallbackInstance, DWORD fdwOpen );
static MMRESULT waveInStart( HWAVEIN hwi );
static MMRESULT waveInStop ( HWAVEIN hwi );

/*
static MMRESULT waveInAddBuffer( HWAVEIN hwi, LPWAVEHDR pwh, UINT cbwh );
static MMRESULT waveInUnprepareHeader( HWAVEIN hwi, LPWAVEHDR pwh, UINT cbwh );
static MMRESULT waveInPrepareHeader( HWAVEIN hwi, LPWAVEHDR pwh, UINT cbwh );
static void CALLBACK waveInProc( HWAVEIN hwi,UINT uMsg, DWORD dwInstance,DWORD dwParam1, DWORD dwParam2 );
static MMRESULT waveInReset( HWAVEIN hwi );
static MMRESULT waveInGetErrorText( MMRESULT mmrError,  LPSTR pszText, UINT cchText );
static MMRESULT waveInGetID( HWAVEIN hwi, LPUINT puDeviceID );
static MMRESULT waveInGetPosition( HWAVEIN hwi, LPMMTIME pmmt, UINT cbmmt );
static DWORD    waveInMessage( HWAVEIN deviceID, UINT uMsg, DWORD_PTR dwParam1, DWORD_PTR dwParam2 );*/

/* ------ redirection ------ */

struct qmic_voice
{
    struct audio_driver * driver;
    SpeexCodec 		* codec;
    HWAVEIN 	 	handle;
	uint32		 	buffer_number;
};

#define QMIC_MAX_VOICES 32

static struct qmic_voice voices[QMIC_MAX_VOICES];

#ifdef WITH_DEBUG_QMIC
static char * get_msg_name ( uint16 msg_id )
{
    static char msg[128];
    switch ( msg_id )
    {
	case QMIC_CMD_HANDSHAKE : strcpy( msg , "Handshake" ); break;
	case QMIC_CMD_GOODBYE   : strcpy( msg , "Goodbye" ); break;
	case QMIC_CMD_ENABLE_COMPRESSION: strcpy( msg , "Enable compression"); break;
	case WIDM_GETNUMDEVS: strcpy( msg , "Get num devs"); break;
        case WIDM_GETDEVCAPS: strcpy( msg , "Get dev caps"); break;
        case WIDM_OPEN: strcpy( msg , "Open"); break;
	case WIDM_CLOSE: strcpy( msg , "Close"); break;
        case WIDM_PREPARE: strcpy( msg, "Prepare"); break;
	case WIDM_UNPREPARE: strcpy( msg, "Unprepare"); break;
        case WIDM_ADDBUFFER: strcpy( msg, "Add buffer"); break;
	case WIDM_START: strcpy( msg ,"Start"); break;
        case WIDM_STOP: strcpy ( msg ,"Stop"); break;
        case WIDM_RESET: strcpy( msg , "Reset"); break;
        case WIDM_GETPOS: strcpy( msg, "Get position"); break;
        case WIDM_PREFERRED: strcpy( msg , "Preferred"); break;
	default: sprintf(msg , "QMIC: Unknown message id : '%d -> 0x%08x'", (int)msg_id, (int)msg_id);
    }
    return msg;
}

static char * get_msg_type_name ( uint32 msg_type_id )
{
    static char msg[128];
    switch ( msg_type_id )
    {
        case QMIC_MT_INTERNAL: strcpy( msg , "Internal message" ); break;
        case QMIC_MT_WID     : strcpy( msg , "WID message" ); break;
        default: sprintf(msg, "QMIC: Unknown message type id : '%d -> 0x%08x'", (int)msg_type_id, (int)msg_type_id);
    }
    return msg;
}

#endif

int qmic_handle_internal_message ( uint16 dev_id, uint16 msg, STREAM s, STREAM response )
{    
    switch ( msg )
    {
	case QMIC_CMD_HANDSHAKE:
	{
	    uint32 days;
	    in_uint32_le( s , days );
	    
	    DEBUG_QMIC(("    Handshake data:\n"));
	    DEBUG_QMIC(("      Days : %d\n", days));
	    
	    out_uint32_le( response , MMSYSERR_NOERROR );
	    out_uint16_le( response , QMIC_RPS_HANDSHAKE );
	    out_uint32_le( response , days );	    
	}break;
	
	case QMIC_CMD_GOODBYE:
	{	
	    DEBUG_QMIC(("    No additional data\n"));
	    
	    out_uint32_le( response , MMSYSERR_NOERROR );
	    out_uint16_le( response , QMIC_RPS_GOODBYE );
	}break;
	
	case QMIC_CMD_ENABLE_COMPRESSION:
	{
	    uint8 compr;
	    in_uint8( s,compr );
	    
	    DEBUG_QMIC(("    Enable compression data:\n"));
	    DEBUG_QMIC(("      IsEnabled : %c\n", compr + '1'));
	    
	    qmic_compression = compr & qmic_compression_support;
	    
	    out_uint32_le ( response , MMSYSERR_NOERROR );
	    out_uint8 ( response , qmic_compression );	    
	}break;
	default :
	{
	    DEBUG_QMIC(("QMIC: Unknown message '%d -> 0x%08x'\n", (int)msg , (int)msg));
	    out_uint32_le( response , MMSYSERR_ERROR );
	}break;
    }
    return QMIC_OK;
}

struct qmic_voice * qmic_find_voice_by_handle ( uint32 h_wave_in )
{
    uint32 i;
    for ( i = 0; i < QMIC_MAX_VOICES; ++ i )
    {
		if ( voices[i].handle == 0 ) continue;
		if ( voices[i].handle == h_wave_in )
		{
		    return &voices[i];
		}
	}
    return NULL;
}

RD_BOOL qmic_have_active_voice ()
{
	uint32 i;
	for ( i = 0; i < QMIC_MAX_VOICES; ++i )
	{
		if ( voices[i].handle != 0 )
			return True;
	}
	return False;
}

int qmic_handle_wid_message ( uint16 dev_id, uint16 msg , STREAM s, STREAM response )
{
    uint32 h_wave_in;
    struct qmic_voice * voice;

    switch ( msg )
    {
    case WIDM_GETNUMDEVS:
    {
		DEBUG_QMIC(("    No additional data\n"));

		uint32 num_devices = waveInGetNumDevs();
		
		out_uint32_le ( response , num_devices );
		
		return QMIC_OK;
    }break;
    case WIDM_GETDEVCAPS:
    {
		uint32 caps_size;
		void * caps_struct;
		in_uint32_le ( s , caps_size );
		
		DEBUG_QMIC(("    Get dev caps data:\n"));
		DEBUG_QMIC(("      Caps size : %d\n", caps_size));
		
		caps_struct = xmalloc(caps_size);
		
		{
		    MMRESULT result = waveInGetDevCaps( dev_id , caps_struct, caps_size );
	    	
		    out_uint32_le ( response , result );
		    out_uint8p ( response , caps_struct, caps_size );
		}
		
		xfree( caps_struct );
	
	return QMIC_OK;
    }break;
    case WIDM_OPEN:
    {
		uint32 format_size;
		WAVEFORMATEX * format;
		uint32 flags;
		unsigned char * format_p;		

		in_uint32_le ( s , format_size );
		in_uint8p    ( s , format_p , format_size );
		format = (WAVEFORMATEX*)format_p;
		in_uint32_le ( s , flags);
		
		DEBUG_QMIC(("    Open data:\n"));
		DEBUG_QMIC(("      Format size : %d\n", format_size));
		DEBUG_QMIC(("      WAVEFORMATEX structure\n"));
		DEBUG_QMIC(("      Flags       : %d\n", flags));
		
		{
		    HWAVEIN hin = 0;
		    MMRESULT result = waveInOpen(&hin, dev_id, format , 0, 0, flags );	
		    out_uint32_le ( response , result );
		    out_uint32_le ( response , hin    );
		    out_uint32_le ( response , hin    ); /* HWAVEIN */
		}
		
		return QMIC_OK;			
    }break;
    }/* switch */    
    
    in_uint8s(s, 4);    
    in_uint32_le( s, h_wave_in );    
		
    DEBUG_QMIC(("    Data:\n"));	
    DEBUG_QMIC(("      HWAVEIN : 0x%08x\n", h_wave_in));
    
    voice = qmic_find_voice_by_handle ( h_wave_in );
    
    if ( voice == NULL )
    {
		DEBUG_QMIC(("QMIC: Can't find voice by handle '%d -> 0x%08x'", (int) h_wave_in, (int)h_wave_in ));
		out_uint32_le ( response , MMSYSERR_INVALHANDLE);
		return QMIC_OK;
    }
    
    
    switch ( msg ) 
    {    
    case WIDM_CLOSE    :
    {
		MMRESULT result = waveInClose( h_wave_in );
		out_uint32_le ( response , result );
		return QMIC_OK;
    }break;
    case WIDM_START    :
    {
		MMRESULT result = waveInStart ( h_wave_in );
		rec_device_open = (result == MMSYSERR_NOERROR)? True : rec_device_open;
		out_uint32_le ( response , result );
		return QMIC_OK;
    }
    case WIDM_STOP     :
    {
		MMRESULT result = waveInStop ( h_wave_in );
		rec_device_open = qmic_have_active_voice();
		out_uint32_le ( response , result );
		return QMIC_OK;
    }break;
    case WIDM_PREPARE  : /* no redirection */
    case WIDM_UNPREPARE:
    case WIDM_ADDBUFFER:
    case WIDM_RESET    :
    case WIDM_GETPOS   :
    case WIDM_PREFERRED:
    {
		out_uint32_le( response , MMSYSERR_NOERROR);
		return QMIC_OK;
    }break; 
    default:
    {
		DEBUG_QMIC(("QMIC: Unknown WID message '%d -> 0x%08x'\n" , (int)msg , (int)msg) );
		out_uint32_le( response , MMSYSERR_ERROR);
		return QMIC_FAIL;
    }
    }/* switch */

    return QMIC_FAIL;
}

STREAM init_response ( VCHANNEL * channel )
{
    return channel_init ( channel , 65536 );
}


void qmic_cmd_process ( STREAM s )
{
	/*
		All internal messages starts with
		uint32  request_number
		uint32  msg_type
		uint16  msg
		uint16  dev_id		
	*/
	STREAM response;

	uint32 request;
	uint16 dev_id;
	uint16 msg;
	uint32 msg_type;

	response = init_response ( qmic_cmd_channel );
	
	DEBUG_QMIC(("QMIC : cmd_channel process\n\n" ) );
	while (!s_check_end( s ) )
	{
		int handler_error = 0;
	
#ifdef WITH_DEBUG_QMIC
		hexdump( s->p, s->end - s->p );
#endif
		
		if ( s->end - s->p < 12 ) /* HDR corruped */
		    break;
		
		in_uint32_le( s, request  );
		in_uint32_le( s, msg_type );
		in_uint16_le( s, msg      );
		in_uint16_le( s, dev_id   );
		
		DEBUG_QMIC(("\n\npacket header:\n"));
		DEBUG_QMIC(("    request #: %d\n", request ));
		DEBUG_QMIC(("    dev_id   : %d\n", dev_id ));
		DEBUG_QMIC(("    msg_type : %s\n", get_msg_type_name( msg_type )));
		DEBUG_QMIC(("    msg      : %s\n", get_msg_name( msg )));
		
		out_uint32_le ( response , request );
		
		switch ( msg_type )
		{
		    case QMIC_MT_INTERNAL:
				handler_error = qmic_handle_internal_message( dev_id, msg , s , response );
			break;
		    case QMIC_MT_WID:
				handler_error = qmic_handle_wid_message ( dev_id, msg, s , response );
			break;
		    default:
		        handler_error = QMIC_FAIL;
				error ("QMIC: Unknown message type '%d -> 0x%08x'\n" , (int) msg_type , (int)msg_type );
		    /* error */
		}
		DEBUG_QMIC(("\n\n"));
		DEBUG_QMIC(("QMIC: Handler returned : %d\n\n", handler_error ));
		
		s_mark_end(response);
		channel_send( response, qmic_cmd_channel );	
		DEBUG_QMIC(("QMIC: !!!!\n"));
					
	}
	DEBUG_QMIC( ("\n\nQMIC : cmd_channel -------\n\n" ) );
}

void qmic_data_process ( STREAM s )
{
	DEBUG_QMIC( ("QMIC : snd_channel process\n\n" ) );
	while (!s_check_end( s ) )
	{
#ifdef WITH_DEBUG_QMIC
		hexdump( s->p, s->end - s->p );
#endif
		s->p = s->end;
	}
	DEBUG_QMIC( ("\n\nQMIC : snd_channel -------\n\n" ) );
}


RD_BOOL qmic_init ( char * _options )
{
	struct audio_driver * snd_driver = qmic_get_current_snd_driver();
	DEBUG_QMIC(("QMIC: init qmic\n" ));

	if ( snd_driver == NULL )
	{
	    DEBUG_QMIC(("QMIC: Sound device not selected\n"));
	}		

	qmic_cmd_channel = 
		channel_register( "MICCMDC", CHANNEL_OPTION_INITIALIZED | CHANNEL_OPTION_ENCRYPT_RDP | CHANNEL_OPTION_SHOW_PROTOCOL,
				 qmic_cmd_process);

	qmic_data_channel = 
		channel_register ("MICSNDC", CHANNEL_OPTION_INITIALIZED | CHANNEL_OPTION_ENCRYPT_RDP | CHANNEL_OPTION_SHOW_PROTOCOL,
				 qmic_data_process);

	if ( (qmic_cmd_channel == NULL) || (qmic_data_channel == NULL) )
	{
		error ( "channel_register\n");
		return False;
	}

	/* initialize voices */
	{
	    uint32 i;
	    for ( i = 0; i < QMIC_MAX_VOICES; ++i )
	    {
			voices[i].driver = NULL;
			voices[i].codec  = NULL;
			voices[i].handle = 0;
			voices[i].buffer_number = 0;
	    }
	}	

	return True;
}

#include <math.h>

void qmic_record_voice ( struct qmic_voice * voice , char * buffer , int size )
{
	STREAM response;
	uint8 compressed [ 65636 ];
	uint32 compressed_size;

	if ( qmic_compression == True )
	{
		compressed_size = codec_compress_buffer ( voice->codec,  buffer , (uint32)size , compressed );

#ifdef WITH_DEBUG_QMIC
		hexdump ( compressed , compressed_size );
#endif
	}
	else
	{
		memcpy ( compressed , buffer , size );
		compressed_size = size;
	}

	response = init_response ( qmic_data_channel );

	out_uint32_le(response , voice->handle );
	out_uint32_le(response , voice->handle );
	out_uint16_le(response , 0x03C0); /*0x03C0 - WIM_DATA message*/
	out_uint32_le(response , voice->buffer_number );
	out_uint32_le(response , size );
	out_uint32_le(response , 0x0001 | 0x0002); /* 0x0001 - buffer ready for application use (done) 0x0002 - prepared */
	out_uint8p( response , compressed, compressed_size );

	s_mark_end( response );

	channel_send( response , qmic_data_channel );

	++voice->buffer_number;	
}


void qmic_record ( char * buffer , int size )
{
	int i;

	for ( i = 0; i < QMIC_MAX_VOICES; ++i )
	{
		if ( voices[i].handle == 0 ) continue;

		qmic_record_voice ( &voices[i] , buffer, size );
	}

}

void strcpyw( short * dst , const char * src )
{
    char * cdst = (char*)dst;
    
    while ( *src )
    {
	*cdst++ = *src++;
	*cdst++ = 0;
    }
    *cdst ++ = 0;
    *cdst ++ = 0;
}

UINT     waveInGetNumDevs( void )
{    
    return 1;
}

MMRESULT waveInGetDevCaps( UINT_PTR uDeviceID, LPWAVEINCAPS pwic, UINT cbwic )
{
    DEBUG_QMIC(("waveInGetDevCaps\n"));
    
    if ( cbwic == sizeof ( WAVEINCAPSA ) )
    {
	DEBUG_QMIC(("ANSI version\n"));
	LPWAVEINCAPSA wca = (LPWAVEINCAPSA) pwic;
	wca->wMid = wca->wPid = 0;
	wca->vDriverVersion = 1 << 16;
	strcpy ( wca->szPname, "RDP Microphone" );
	wca->dwFormats = 0x00000800; /* 44.1 kHz , stereo, 16 bps */
	wca->wChannels = 2;
	wca->wReserved1 = 0;
    }
    else
    {
	DEBUG_QMIC(("UNICODE version, pwic %p\n", pwic));
	LPWAVEINCAPSW wcw = (LPWAVEINCAPSW) pwic;
	wcw->wMid = wcw->wPid = 0;
	wcw->vDriverVersion = 1 << 16;
	strcpyw ( wcw->szPname, "RDP Microphone" );
	wcw->dwFormats = 0x00000800; /* 44.1 kHz , stereo, 16 bps */
	wcw->wChannels = 2;
	wcw->wReserved1 = 0;		
    }
    
    return MMSYSERR_NOERROR;
}

struct qmic_voice * qmic_get_new_voice ()
{
    uint32 i;
    for ( i = 0; i < QMIC_MAX_VOICES; ++i )
    {
		if ( voices[i].handle == 0 ) return &voices[i];
    }
    return NULL;
}

MMRESULT waveInOpen( LPHWAVEIN phwi, UINT_PTR uDeviceID, LPWAVEFORMATEX pwfx,DWORD_PTR dwCallback, DWORD_PTR dwCallbackInstance, DWORD fdwOpen )
{
    DEBUG_QMIC(("QMIC: waveInOpen\n"));
    
    struct audio_driver * snd = qmic_get_current_snd_driver();
    
    if ( snd == NULL )
    {
	DEBUG_QMIC(("QMIC: Sound recorder not selected\n"));
	return MMSYSERR_NODRIVER; 
    }
    
    if ( fdwOpen & 0x0001 ) /* Query open ?*/
    {
		DEBUG_QMIC(("QMIC: phwi %p\n", phwi ));
		*phwi = 0; /* no handle will be created */
		
		DEBUG_QMIC (("QMIC: snd : %p , snd->wave_in_format_supported %p\n", snd , ((snd)? snd->wave_in_format_supported: NULL) ));
		
		return ( snd->wave_in_format_supported ( pwfx ) == True )? MMSYSERR_NOERROR : 32; /*32 - WAVERR_BADFORMAT*/
    }
    else
    {
	/* create codec and return id as handle to wavein device */
	
		struct qmic_voice * voice = qmic_get_new_voice();

		if ( voice == NULL )
		{
		    DEBUG_QMIC(("QMIC: can't create new voice!\n"));
		    *phwi = 0;
		    return MMSYSERR_ALLOCATED;
		}
		
		while ( voice->handle == 0 )
		    voice->handle = rand();

		voice->driver = snd;
		voice->codec  = codec_create ( pwfx , 3 , 5 );
		voice->buffer_number = 0;
		
		*phwi = voice->handle;
		
		DEBUG_QMIC(("QMIC: voice created , handle 0x%08x, driver %p, codec %p\n" , voice->handle, voice->driver, voice->codec ));
    }
    
    return MMSYSERR_NOERROR;
}

MMRESULT waveInClose( HWAVEIN hwi )
{
    struct qmic_voice * voice = qmic_find_voice_by_handle ( hwi );
    if ( voice == NULL )
	return MMSYSERR_INVALHANDLE;


    voice->handle = 0;
    voice->driver->wave_in_close();
    voice->driver = NULL;
    
    codec_destroy ( voice->codec );
    voice->codec = NULL;
	voice->buffer_number = 0;

    
    return MMSYSERR_NOERROR;
}

MMRESULT waveInStart( HWAVEIN hwi )
{
    RD_BOOL result;
    struct qmic_voice * voice = qmic_find_voice_by_handle ( hwi );
    if ( voice == NULL )
	return MMSYSERR_INVALHANDLE;

    DEBUG_QMIC(("QMIC:Opening device..."));
    result = voice->driver->wave_in_open();
    DEBUG_QMIC(("Ok [%d]\nQMIC: setting format...",result));
    voice->driver->wave_in_set_format ( &voice->codec->format );
    DEBUG_QMIC(("Ok\n"));	
    
    return (result == True )? MMSYSERR_NOERROR : MMSYSERR_ERROR;
}

MMRESULT waveInStop ( HWAVEIN hwi )
{
    struct qmic_voice * voice = qmic_find_voice_by_handle ( hwi );
    if ( voice == NULL )
	return MMSYSERR_INVALHANDLE;
	
    voice->driver->wave_in_close ();
    
    return MMSYSERR_NOERROR;
}

